package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.ObjectMap;
import com.rube.Box2dLoader;
import com.rube.Drawer;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.LevelData;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.objects.Ball;
import com.zyb.loveball.objects.GameBg;
import com.zyb.loveball.objects.Hint;
import com.zyb.loveball.objects.Scene;
import com.zyb.loveball.objects.Zombie;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.Matrix4s;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class GamePanel extends Group {
    OrthographicCamera camera;
    float cameraHeight = -1;
    PaintPanel paintPanel;
    World world;
    Array<Body> bodies = new Array<>();
    Drawer drawer;
    Box2dLoader box2dLoader;
    Scene scene;
    Set<Body> bodyRemoveList = new HashSet<>();
    ArrayMap<Body, Boolean> bodyActiveList = new ArrayMap<>();
    ArrayMap<Body, Vector3> bodyTransformList = new ArrayMap<>();
    ArrayMap<Body, FixtureDef> bodyCreateFixtureList = new ArrayMap<>();
    ArrayMap<Body, Fixture> bodyDestoryFixture = new ArrayMap<>();
    Set<Body> objectRemoveList = new HashSet<>();
    GameState state = GameState.ready;
    private GameState oldState = GameState.ready;
    GameBg bg;
    protected Box2DDebugRenderer renderer; // 测试用绘制器
    private UIPanel gameUI;
    PaintController paintController;
    boolean success = false;
    LevelData levelData;
    boolean slowMotion;
    boolean cameraZoomIn;
    protected Hint hint;
    Vector2 lastZombieBoomPosition;

    public GamePanel(float width, float height, UIPanel uiPanel) {
        this.levelData = GameInfo.levelData;
        this.gameUI = uiPanel;
        initCamera(width, height);
        initGame();
        GameTime.reset();
        if (GameInfo.hint || GameInfo.levelData.isShowHint()) {
            hint();
        }
    }

    protected void initGame() {
        world = new World(new Vector2(0, -9.8f), true); // 标准重力场
        World.setVelocityThreshold(0);
        drawer = new Drawer();
        box2dLoader = new Box2dLoader();
        box2dLoader.setHook(box2dLoader.getHook());
        bg = new GameBg(camera);
        paintPanel = new PaintPanel(this);
        scene = new Scene(this);
        world.getBodies(bodies);
//        setWorldActive(false);
        world.setContactListener(new WorldContactHandler());
        if (Configuration.box2dDebug) {
            renderer = new Box2DDebugRenderer();
        }
        paintController = new PaintController();
        System.out.println("paintController!!");
        hint = paintPanel.hint;
    }

    @Override
    protected void setParent(Group parent) {
        super.setParent(parent);
        if (parent == null) return;
        parent.addActor(paintController);
        paintController.setListener(new PaintController.PaintControllerListener() {
            @Override
            public void touchDown(float x, float y) {
                paintPanel.start(x, y);
            }

            @Override
            public void touchDragged(float x, float y) {
                paintPanel.painting(x, y);
            }

            @Override
            public void touchUp(float x, float y) {
                paintPanel.end(x, y);
            }
        });
    }

    private void initCamera(float width, float height) {
        camera = new OrthographicCamera(width, height);
        camera.position.set(camera.viewportWidth / 2, camera.viewportHeight / 2, 0);
        if (GameScreen.gameType == GameScreen.GameType.NORMAL) {
            camera.zoom = 1;
        } else if (GameScreen.gameType == GameScreen.GameType.HAPPY_GLASS) {
            camera.zoom = Constant.GAME_CAMERA_ZOOM;
        }
        camera.update();
        Matrix4s.game.set(camera.combined);
    }


    public PaintPanel getPaintPanel() {
        return paintPanel;
    }

    public World getWorld() {
        return world;
    }

    Vector3 tmpVector3 = new Vector3();

    //    Vector2 tmpVector2 = new Vector2();
    public void updateWorldPosition(Vector2 position) {
//        tmpVector3 = camera.project(tmpVector3.set(position.x,position.y,0));
//        gameUI.updateWorldPosition(tmpVector2.set(tmpVector3.x*Constant.SCALE,tmpVector3.y*Constant.SCALE));
        gameUI.updateWorldPosition(position);
////        magnifierUtil.update(position);
    }

    public void drawMagnifier(Batch batch) {
        bg.draw(batch, 1);
        scene.draw(batch, 1);
        paintPanel.draw(batch);
    }

    //判断是否重叠，重叠则延长线段到焦点
    public boolean isInBodies(float x, float y) {
        for (Body body : bodies) {
            boolean active = body.isActive();
            if (!active) continue;
            for (Fixture fixture : body.getFixtureList()) {
                if (fixture.testPoint(x, y)) {
                    if (scene.getName(body) == null || !scene.getName(body).equals("fan")) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

//    public void setWorldActive(boolean isActive){
//        String name;
//        for(Body body:bodies){
//            name = scene.getName(body);
//            if(name == null || !name.equals("move_platform_dynamic")){
//                body.setActive(isActive);
//            }
//        }
//    }

    public Body createBody(BodyDef bodyDef) {
        Body body = world.createBody(bodyDef);
        bodies.add(body);
        return body;
    }

    public Box2dLoader getBox2dLoader() {
        return box2dLoader;
    }

    public void failure() {
        Log.log("GamePanel", "failure");
        if (state == GameState.failure) return;
        state = GameState.failure;
        gameUI.failure(false);
    }


    public void end(boolean success, float endDelay) {
        if (state == GameState.end) return;
        Log.log("GamePanel", "end()");
        HelloZombieGame.pauseInputProcessor();
        this.success = success;
        state = GameState.end;
        if (success) {
            gameUI.hideButtons();
            if(GameInfo.type==0) {
                cameraZoomIn = true;
            }
            int ballId = Configuration.settingData.getBallId();
            if (ballId == Constant.BOMB_ID_1 || ballId == Constant.BOMB_ID_2) {
                GameTime.endDelay = 2.3F;
            } else {
                if(GameInfo.type==0) {
                    slowMotion = true;
                }
                GameTime.endDelay = 1.5F;
            }
            gameUI.screenShot();
        } else {
            GameTime.endDelay = 1.5F;
        }
        if (endDelay != -1) {
            GameTime.endDelay = endDelay;
        }
        gameUI.end();
        paintPanel.end();
    }

    public void end(boolean success) {
        end(success, -1);
    }

    public void endNoDelay(boolean success) {
        end(success, 0.1f);
    }

    private void success() {
        if (state == GameState.success) return;
        state = GameState.success;
        gameUI.success();
        Log.log("GamePanel", "success");
    }

    public void destroyBody(Body body) {
        if (body == null) return;
        if (!world.isLocked()) {
            world.destroyBody(body);
        } else {
            bodyRemoveList.add(body);
        }
    }

    public void destoryFixture(Body body, Fixture fixture) {
        if (!world.isLocked()) {
            body.destroyFixture(fixture);
        } else {
            bodyDestoryFixture.put(body, fixture);
        }
    }

    public void activeBody(Body body, boolean active) {
        if (!world.isLocked()) {
            body.setActive(active);
        } else {
            bodyActiveList.put(body, active);
        }
    }

    public void createFixture(Body body, FixtureDef fixtureDef) {
        if (!world.isLocked()) {
            body.createFixture(fixtureDef);
        } else {
            bodyCreateFixtureList.put(body, fixtureDef);
        }
    }

    public Drawer getDrawer() {
        return drawer;
    }

    float deltaScale = 1;

    public void updateGame(float delta) {
        if (slowMotion) {
//            System.out.println(delta);
//            if(delta>=0.016f&&delta<=0.018f) {
//                deltaScale *= delta / 0.016f;
//            }else{
//                deltaScale *= 1.03f;
//            }
//            delta/=deltaScale;
//            if(deltaScale>=7f) slowMotion = false;
            delta /= 4;
            if (GameTime.unActiveTime >= 0.3f) slowMotion = false;
        }
        updateWorld(delta);
        hint.updateHint();
        updateState(delta);
        scene.updateUI(delta);
    }

    float stepTime;
    float startTime = 0;
    int velocityIterations = 8;
    int positionIterations = 3;

    private void updateWorld(float delta) {
        if (state == GameState.gaming || state == GameState.end) {
            preprocess();
            stepTime += delta;
            float TIME_STEP = Constant.TIME_STEP;
            if (slowMotion) {
                TIME_STEP = Constant.TIME_STEP_BULLET;
            }
            if (stepTime > 2 * TIME_STEP) {
                velocityIterations = 4;
                positionIterations = 1;
            } else {
                velocityIterations = 8;
                positionIterations = 3;
            }
            while (stepTime >= TIME_STEP) {
                stepTime -= TIME_STEP;
                world.step(TIME_STEP, velocityIterations, positionIterations);
                scene.updateWorld(TIME_STEP);
            }

        }
//        System.out.println(state);
    }

    private void preprocess() {
        Iterator<Body> iterator = bodyRemoveList.iterator();
        while (iterator.hasNext()) {
            Body body = iterator.next();
            bodies.removeValue(body, true);
            body.setContactListener(null);
            world.destroyBody(body);
        }
        Iterator<ObjectMap.Entry<Body, Boolean>> iterator1 = bodyActiveList.iterator();
        while (iterator1.hasNext()) {
            ObjectMap.Entry<Body, Boolean> entry = iterator1.next();
            entry.key.setActive(entry.value);
        }

        Iterator<ObjectMap.Entry<Body, Vector3>> iterator2 = bodyTransformList.iterator();
        while (iterator2.hasNext()) {
            ObjectMap.Entry<Body, Vector3> entry = iterator2.next();
            entry.key.setTransform(entry.value.x, entry.value.y, entry.value.z);
        }

        Iterator<ObjectMap.Entry<Body, FixtureDef>> iterator3 = bodyCreateFixtureList.iterator();
        while (iterator3.hasNext()) {
            ObjectMap.Entry<Body, FixtureDef> entry = iterator3.next();
            entry.key.createFixture(entry.value);
        }

        Iterator<ObjectMap.Entry<Body, Fixture>> iterator4 = bodyDestoryFixture.iterator();
        while (iterator4.hasNext()) {
            ObjectMap.Entry<Body, Fixture> entry = iterator4.next();
            entry.key.destroyFixture(entry.value);
        }
        Iterator<Body> iterator5 = objectRemoveList.iterator();
        while (iterator5.hasNext()) {
            Body body = iterator5.next();
            paintPanel.removeObject(body);
            scene.removeObject(body);
            destroyBody(body);
        }
        bodyActiveList.clear();
        bodyRemoveList.clear();
        bodyTransformList.clear();
        bodyCreateFixtureList.clear();
        bodyDestoryFixture.clear();
        objectRemoveList.clear();
    }


    public void removeObject(Body body) {
        objectRemoveList.add(body);
    }

    private void updateState(float delta) {
        float oldGameTime = GameTime.gameTime;
        if (state == GameState.gaming) {
            GameTime.gameTime += delta;
        }
        if (GameScreen.gameType == GameScreen.GameType.HAPPY_GLASS) {
            if (Constant.HAPPYGLASS_SECONDS - oldGameTime >= 3 && Constant.HAPPYGLASS_SECONDS - GameTime.gameTime <= 3) {
                gameUI.countdown(true, 3);
            }
        }
        if (!paintPanel.painting) {
            GameTime.unActiveTime += delta;
        }
        if (gameUI != null && gameUI.noviceCourse != null) {
            gameUI.noviceCourse.noviceCourseCheck(GameTime.unActiveTime, state);
        }

        if (GameTime.unActiveTime > Constant.handSecond && !GameInfo.hint && (state == GameState.ready || state == GameState.gaming)) {
            gameUI.showHand(true);
        } else {
            if (gameUI != null) {
                gameUI.showHand(false);
            }
        }
        if (!GameInfo.levelData.isShowHint() && GameInfo.failureTimes != 0 && hint.isShowHint() && (state == GameState.gaming || state == GameState.ready)) {
            GameTime.hintTime += delta;
            if (GameTime.hintTime > 3) {
                gameUI.showSkip(true);
                GameTime.hintTime = 0;
            }
        }
        if (state == GameState.end) {
            GameTime.stateTime += delta;
            if (GameTime.stateTime >= GameTime.endDelay) {
                if (success) {
                    success();
                } else {
                    failure();
                }
                GameTime.stateTime = 0;
            }
        }
    }


    public void transformBody(Body body, float x, float y, float rotation) {
        bodyTransformList.put(body, new Vector3(x, y, rotation));
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        float deltaTime = Gdx.graphics.getDeltaTime();
        updateGame(deltaTime);
        batch.setProjectionMatrix(camera.combined);
        if (cameraZoomIn) {
            zoomIn(deltaTime);
        }
        bg.draw(batch, parentAlpha);
        scene.draw(batch, parentAlpha);
        paintPanel.draw(batch);
        if (paintPanel.painting) {
            if (gameUI != null)
                gameUI.magnifier(true);
        } else {
            if (gameUI != null)
                gameUI.magnifier(false);
        }
        batch.setProjectionMatrix(Matrix4s.ui);
        if (Configuration.box2dDebug) {
            box2dDebug(batch);
        }
    }

    protected void zoomIn(float deltaTime) {
        float targetX = 4.5f;
        float targetY = 5f;
        float primaryX = 3f;
        float primaryY = 5f;
        float primaryZoom = 1f;
        float targetZoom = 0.5f;
        float inTime = 0.2f;
        if (GameInfo.type == 0) {
            primaryZoom = 1;
        } else if (GameInfo.type == 1) {
            primaryZoom = 2;
        }
        float x1 = 3 - primaryZoom * 3;
        float y1 = 5 - primaryZoom * 5 - (primaryZoom - 1) * (getCamera().viewportHeight - 10) / 2;
        float x2 = x1 + getCamera().viewportWidth * primaryZoom;
        float y2 = y1 + getCamera().viewportHeight * primaryZoom;
        targetX = Math.max(Math.min(x2 - 1.5f, getLastZombieBoomPosition().x), x1 + 1.5f);
        targetY = Math.max(Math.min(y2 - 2.5f, getLastZombieBoomPosition().y + 0.5f), y1 + 2.5f);
        if (camera.zoom > 0.5f) {
            camera.zoom -= (primaryZoom - targetZoom) * deltaTime / inTime;
            camera.translate((targetX - primaryX) * deltaTime / inTime, (targetY - primaryY) * deltaTime / inTime);
            camera.update();
        } else {
            cameraZoomIn = false;
        }
    }

    private void box2dDebug(Batch batch) {
        batch.end();
        renderer.render(world, camera.combined);
        batch.begin();
    }

    public void dispose() {
        Log.log("GamePanel", "dispose()");
        scene.dispose();
        if (world != null) {
            world.dispose();
            world = null;
        }
        if (paintController != null) {
            paintController.remove();
        }
        bodies.clear();
        scene = null;
        gameUI = null;
        box2dLoader.dispose();
    }

    public void start() {
        if (state == GameState.ready) {
            state = GameState.gaming;
//            setWorldActive(true);
        }
    }

    public Scene getScene() {
        return scene;
    }


    public boolean pause() {
        if (state == GameState.gaming || state == GameState.ready) {
            oldState = state;
            state = GameState.pause;
            return true;
        }
        return false;
    }

    public GameState getState() {
        return state;
    }

    public boolean resume() {
        if (state == GameState.pause) {
            state = oldState;
            return true;
        }
        return false;
    }

    public void updateUI() {
        gameUI.update();
        if (GameInfo.challenge) {
            if (!paintPanel.pen.canWrite()) {
                gameUI.countdown(true, 5);
            }
//            if(GameInfo.chapter ==1){
//                if(paintPanel.pen.getInkLeft() == 0){
//                    gameUI.countdown(true,5);
//                }
//            }else if(GameInfo.chapter ==3){
//                if(!paintPanel.pen.canWrite()){
//
//                }
//            }
        }
    }


    public OrthographicCamera getCamera() {
        return camera;
    }

    public Vector3 getZombieScreenPosition() {
        Vector2 zombiePosition = scene.getZombie().getPosition();
        Vector3 temp = new Vector3();
        temp.set(zombiePosition, 0);
        temp = camera.project(temp);
        return temp;
    }

    public void setLastZombieBoomPosition(Vector2 position) {
        lastZombieBoomPosition = position;
    }

    public Vector2 getLastZombieBoomPosition() {
        if (lastZombieBoomPosition == null) {
            return new Vector2(0, 0);
        } else {
            return lastZombieBoomPosition;
        }
    }

    public Vector3 getHintLineScreenPosition() {
        Vector2 hintLinePosition = getHintLinePosition();
        Vector3 temp = new Vector3();
        if (hintLinePosition != null) {
            temp.set(hintLinePosition, 0);
        } else {
            temp.set(0, 0, 0);
        }
        temp = camera.project(temp);
        return temp;
    }

    public void hint() {
        hint.hint();
        GameInfo.hint = true;
        GameTime.unActiveTime = 0;
        if (gameUI != null)
            gameUI.showHand(false);
    }

    public GameBg getBg() {
        return bg;
    }

    public static class GameTime {
        static float stateTime;
        static float gameTime;
        static float unActiveTime;
        static float endDelay;
        static float hintTime;

        public static void reset() {
            stateTime = 0;
            gameTime = 0;
            unActiveTime = 0;
            endDelay = 0;
            hintTime = 0;
        }
    }

    public void paintEnd() {
        if (GameInfo.challenge) {
            if (!paintPanel.pen.canWrite()) {
                gameUI.countdown(true, 5);
            }
        }
    }

    public float inkLeftPercent() {
        return paintPanel.pen.inkLeftPercent();
    }

    public float getHintLength() {
        return hint.getHintLength();
    }

    public Zombie getZombie() {
        return scene.getZombie();
    }

    public Ball getBall() {
        return scene.getBall();
    }

    public Vector2 getHintLinePosition() {
        Vector2 vector2 = new Vector2();
        if (hint.isShowHint()) {
            vector2.set(hint.getHintLine().getVertexes().get(0), hint.getHintLine().getVertexes().get(1));
            return vector2;
        } else return null;
    }

    Vector2 tmpVector2 = new Vector2();

    public Vector2 gamePanelCameraPositionToZoom1(Vector2 vector2) {
        tmpVector2.x = 3 - camera.zoom * 3;
        tmpVector2.y = 5 - camera.zoom * 5 - (camera.zoom - 1) * (camera.viewportHeight - 10) / 2;
        tmpVector2.x = (vector2.x - tmpVector2.x) / camera.zoom;
        tmpVector2.y = (vector2.y - tmpVector2.y) / camera.zoom;
        return tmpVector2;
    }

    private long vibrateTime;

    public void vibrate() {
        long currentTime = System.currentTimeMillis();
        if (currentTime - vibrateTime < 200) return;
        vibrateTime = currentTime;
        Gdx.input.vibrate(100);
    }

    public enum GameState {
        ready, pause, gaming, end, success, failure
    }
}
