package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.BaseDialog;
import com.zyb.loveball.dialogs.ChallengeConfirmDialog;
import com.zyb.loveball.dialogs.ChallengeQuitDialog;
import com.zyb.loveball.dialogs.FailureDialog;
import com.zyb.loveball.dialogs.HintConfirmDialog;
import com.zyb.loveball.dialogs.PauseDialog;
import com.zyb.loveball.dialogs.RateDialog;
import com.zyb.loveball.dialogs.SuccessBonusBaseDialog;
import com.zyb.loveball.dialogs.SuccessBonusBoxDialog;
import com.zyb.loveball.dialogs.SuccessBonusCardDialog;
import com.zyb.loveball.dialogs.SuccessBonusTurntableDialog;
import com.zyb.loveball.dialogs.SuccessDialog;
import com.zyb.loveball.objects.NoviceCourse;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.ChallengeScreen;
import com.zyb.loveball.screens.ChapterScreen;
import com.zyb.loveball.screens.LevelScreen;
import com.zyb.loveball.screens.MenuScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.Matrix4s;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.SoundButtonListener;


public class GameScreen extends BaseScreen {
    public static boolean canShowFullAds = false;
    GamePanel gamePanel;
    UIPanel uiPanel;
    SuccessDialog successDialog;
    SuccessBonusBaseDialog successBonusBaseDialog;
    private CircleLayer circleLayer;
    public static GameType gameType = GameType.NORMAL;

    public GameScreen(HelloZombieGame game) {
        super(game);
        TAG = "game";
        inTime = 0.5f;
        canShowFullAds = false;
    }

    @Override
    public void show() {
        super.show();
        GameInfo.start();
        if (GameInfo.challenge) {
            uiPanel = new ChallengeUIPanel(scene, this);
        } else {
            uiPanel = new GameUIPanel(scene, this);
        }
//        if(GameScreen.gameType == GameType.NORMAL) {
//            ((OrthographicCamera)stage.getCamera()).zoom = 1;
//            ((OrthographicCamera)stage.getCamera()).update();
//        }else if(GameScreen.gameType == GameType.HAPPY_GLASS){
//            System.out.println("happy glass");
//            ((OrthographicCamera)stage.getCamera()).zoom = 2;
//            ((OrthographicCamera)stage.getCamera()).update();
//        }
        gamePanel = new GamePanel(stage.getCamera().viewportWidth * Constant.SCALE, stage.getCamera().viewportHeight * Constant.SCALE, uiPanel);
        stage.addActor(gamePanel);
        stage.addActor(scene);
        uiPanel.setGamePanel(gamePanel);
        scene.setTouchable(Touchable.childrenOnly);
        initButtons();
        layer.remove();
        Matrix4s.ui.set(stage.getCamera().combined);
        circleLayer = new CircleLayer(inTime*2);
        Vector3 vector3 = gamePanel.getZombieScreenPosition();
        Vector2 zomPosition = screenToStageCoordinates(vector3.x, vector3.y);
        circleLayer.open(zomPosition.x - 10, zomPosition.y + 50);
        stage.addActor(circleLayer);
//        showFullScreenAd(0);
    }

    @Override
    protected void start() {
        super.start();
//        if(circleLayer!=null){
//            circleLayer.remove();
//        }
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        uiPanel.getRoot().findActor("btn_retry", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if(uiPanel.noviceCourse!=null){
                    if(uiPanel.noviceCourse.courseState == NoviceCourse.CourseState.retry_course){
//                        uiPanel.noviceCourse.courseState = NoviceCourse.CourseState.none;
//                        uiPanel.noviceCourse.update();
                        uiPanel.noviceCourse.setNone();
                    }
                }
                gamePanel.failure();
//                uiPanel.failure(true);
            }
        });

        uiPanel.getRoot().findActor("btn_pause", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                pauseGame();
//                showBonusCardDialog();
//                showBonusBoxDialog();
            }
        });
        if (Configuration.debug) {
            findActor("debug").setVisible(true);
            findActor("btn_pre", Touchable.enabled).addListener(new SoundButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    GameInfo.preLevel();
                    gamePanel.end(false);
                }
            });
            findActor("btn_next", Touchable.enabled).addListener(new ButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    GameInfo.nextLevel(GameInfo.type);
                    gamePanel.end(false);
                }
            });
            findActor("test_hint", Touchable.enabled).addListener(new ButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    gamePanel.hint();
                }
            });
        } else {
            findActor("debug").setVisible(false);
        }
    }

    @Override
    public void pause() {
        Log.log("GameScreen", "pause()");
        pauseGame();
    }

    private void pauseGame() {
        if (state != ScreenState.end && state != ScreenState.show) {
            if (getPendingAction() == PendingAction.NONE && gamePanel.pause()) {
                showPauseDialog();
            }
        }
    }

    private int errorNum = 0;
    public void success(){
        try {
        showSuccessDialog();
        if(Configuration.isShowRate()){
            showRateDialog();
        }
        FlurryUtils.level_Win(GameInfo.chapter, GameInfo.id);
        if (GameInfo.star == 3) {
            FlurryUtils.level_3Stars(GameInfo.chapter, GameInfo.id);
        }
        GameInfo.reset();
        }catch (Exception e){
            e.printStackTrace();
            errorNum ++;
            if(errorNum<2) {
                Gdx.app.postRunnable(new Runnable() {
                    @Override
                    public void run() {
                        success();
                    }
                });
            }else{
                end(MenuScreen.class);
            }
        }
    }

    private void showPauseDialog() {
        PauseDialog pauseDialog = (PauseDialog) showDialog("ui/dialogs/pause.json", PauseDialog.class);
        pauseDialog.setPauseDialogListener(new PauseDialog.PauseDialogListener() {
            @Override
            public void home() {
                if (GameInfo.challenge) {
                    ChallengeQuitDialog confirmDialog = (ChallengeQuitDialog) showDialog("ui/dialogs/quit_challenge.json", ChallengeQuitDialog.class);
                    confirmDialog.setConfirmListener(new ChallengeQuitDialog.ConfirmListener() {
                        @Override
                        public void confirm() {
                            end(MenuScreen.class);
                        }
                    });
                } else {
                    end(MenuScreen.class);
                }
            }

            @Override
            public void menu() {
                if (GameInfo.challenge) {
                    ChallengeQuitDialog confirmDialog = (ChallengeQuitDialog) showDialog("ui/dialogs/quit_challenge.json", ChallengeQuitDialog.class);
                    confirmDialog.setConfirmListener(new ChallengeQuitDialog.ConfirmListener() {
                        @Override
                        public void confirm() {
                            end(ChallengeScreen.class);
                        }
                    });
                } else {
                    end(LevelScreen.class);
                }
            }
        });
    }
    public void showHintConfirmDialog() {
        HintConfirmDialog hintConfirmDialog = (HintConfirmDialog) showDialog("ui/dialogs/hint_confirm.json", HintConfirmDialog.class);
        hintConfirmDialog.setHintConfirmDialogListener(new HintConfirmDialog.HintConfirmDialogListener() {
            @Override
            public void playVideoAds() {
               setPendingAction(BaseScreen.PendingAction.HINT);
                HelloZombieGame.launcherListener.playUnityAds();
                FlurryUtils.level_HintAD(GameInfo.chapter, GameInfo.id);
            }
        });
    }


    @Override
    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass) {
        gamePanel.pause();
        return super.showDialog(attribute, tClass);
    }

    @Override
    public void update() {
        super.update();
        if(uiPanel!=null) {
            uiPanel.update();
        }
    }

    @Override
    protected void out() {
        super.out();
    }

    public void showSuccessDialog() {
        successDialog = (SuccessDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/success.json", SuccessDialog.class);
        successDialog.setSuccessDialogListener(new SuccessDialog.SuccessDialogListener() {
            @Override
            public void next() {
            //    GameScreen.canShowFullAds = false;
                nextTouched();

            }

            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                playVideoReward(action,pendingValue);
            }

            @Override
            public void showCoinDialog() {
                GameScreen.this.showCoinDialog();
            }

            @Override
            public void showSuccessBonusTurntableDialog() {
                GameScreen.this.showBonusTurntableDialog();
            }

            @Override
            public void showSuccessBonusCardDialog() {
                GameScreen.this.showBonusCardDialog();
            }
            @Override
            public void showSuccessBonusBoxDialog() {
                GameScreen.this.showBonusBoxDialog();
            }

            @Override
            public void showChallengeConfirmDialog() {
                ChallengeConfirmDialog dialog = (ChallengeConfirmDialog) showDialog("ui/dialogs/challenge_confirm.json", ChallengeConfirmDialog.class);
                dialog.setChallengeConfirmDialogListener(new ChallengeConfirmDialog.ChallengeConfirmDialogListener() {
                    @Override
                    public void jumpToChallengeLevelScreen() {
                        GameInfo.normal(1);
                        end(LevelScreen.class);
                    }
                });
            }
        });
    }

    protected void nextTouched() {
//        boolean showAd = BaseScreen.showFullScreenAd(0, new FullScreenListener() {
//            @Override
//            public void adClosed() {
//                nextLevel();
//            }
//        });
//        if(!showAd){
            nextLevel();
//        }
    }
    public void nextLevel(){
        if(GameInfo.type == 0) {
            if (!(GameInfo.chapter == Constant.CHAPTER_NUN && GameInfo.id == 10)) {
                GameInfo.nextLevel(GameInfo.type);
                end((GameScreen.class));
            } else {
                end(ChapterScreen.class);
            }
        }else if(GameInfo.type == 1){
            if (!(GameInfo.chapter == Constant.HAPPY_GLASS_CHAPTER_NUN+1000 && GameInfo.id == 10)) {
                GameInfo.nextLevel(GameInfo.type);
                end((GameScreen.class));
            } else {
                end(MenuScreen.class);
            }
        }
    }

    public void showFailureDialog() {
        FailureDialog dialog = (FailureDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/revive.json", FailureDialog.class);
        dialog.setReviveDialogListener(new FailureDialog.ReviveDialogListener() {
            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                if (Configuration.videoAdReady) {
                    setPendingAction(action);
                    GameScreen.this.pendingValue = pendingValue;
                    HelloZombieGame.launcherListener.playUnityAds();
                } else {
                    showMessage("The video ad is not ready!");
                }
            }
        });
    }

    public void showRateDialog() {
        RateDialog dialog = (RateDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/rate.json", RateDialog.class);
    }

    @Override
    public void resume() {
        Log.log("GameScreen", "resume()");
        super.resume();
        if (dialogs.size() == 0) {
            gamePanel.resume();
        }
    }

    @Override
    protected void removeDialog() {
        super.removeDialog();
        if (dialogs.empty()) {
            gamePanel.resume();
        }
    }

    @Override
    public void handlePendingAction(boolean done) {
        Log.log("GameScreen", "handlePendingAction()");
        super.handlePendingAction(done);
        switch (getPendingAction()) {
            case HINT:
                gamePanel.hint();
                FlurryUtils.Ads_Reward("Hint");
                setPendingAction(PendingAction.NONE);
                break;
            case RESTART:
                end((GameScreen.class));
                setPendingAction(PendingAction.NONE);
                break;
            case CLEAR:
                gamePanel.paintPanel.clearLines();
                setPendingAction(PendingAction.NONE);
                break;
            case SUCCESS_BONUS:
                if(pendingValue == 2) {
                    showBonusTurntableDialog();
                }else if(pendingValue == 3){
                    showBonusCardDialog();
                }else if(pendingValue == 4){
                    showBonusBoxDialog();
                }else {
                    showBonusTurntableDialog();
                }
                break;
            case SECOND_BONUS:
                secondBonus();
                break;
        }
    }

    protected void secondBonus() {
        setPendingAction(PendingAction.NONE);
        if(successBonusBaseDialog!=null && successBonusBaseDialog instanceof SuccessBonusTurntableDialog){
            FlurryUtils.Ads_Reward("Wheel");
            successBonusBaseDialog.begin();
        }else if(successBonusBaseDialog!=null && successBonusBaseDialog instanceof SuccessBonusCardDialog){
            FlurryUtils.Ads_Reward("Card");
            ((SuccessBonusCardDialog) successBonusBaseDialog).flopAll();
        }else if(successBonusBaseDialog!=null && successBonusBaseDialog instanceof SuccessBonusBoxDialog){
            FlurryUtils.Ads_Reward("Box");
            ((SuccessBonusBoxDialog) successBonusBaseDialog).flopAll();
        }

    }

    protected void showBonusTurntableDialog(){
        setPendingAction(PendingAction.NONE);
        FlurryUtils.Ads_Reward("Wheel");
        SuccessBonusTurntableDialog dialog = (SuccessBonusTurntableDialog) showDialog("ui/dialogs/turntable.json",SuccessBonusTurntableDialog.class);
        successBonusBaseDialog = dialog;
        if(successDialog!=null) {
            successDialog.setVisible(false);
        }
        dialog.setSuccessBonusBaseDialogListener(new SuccessBonusBaseDialog.successBonusBaseDialogListener() {
            @Override
            public void showSuccessDialog() {
                if(successDialog!=null) {
                    successDialog.setVisible(true);
                    successDialog.showAnimation();
                }
            }

            @Override
            public void next() {
                nextTouched();
            }

            @Override
            public CommonAnimation coinGot(int coin) {
                return GameScreen.this.coinGot(coin,true);
            }

            @Override
            public void limitGoodsGot(int id) {
                GameScreen.this.limitGoods(id,true);
            }

            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                playVideoReward(action,pendingValue);
            }
        });
    }
    protected void showBonusCardDialog(){
        setPendingAction(PendingAction.NONE);
        FlurryUtils.Ads_Reward("Card");
        SuccessBonusCardDialog dialog = (SuccessBonusCardDialog)showDialog("ui/dialogs/card_bonus.json", SuccessBonusCardDialog.class);
        successBonusBaseDialog = dialog;
        if(successDialog!=null) {
            successDialog.setVisible(false);
        }
        dialog.setSuccessBonusBaseDialogListener(new SuccessBonusBaseDialog.successBonusBaseDialogListener() {
            @Override
            public void showSuccessDialog() {
                if(successDialog!=null) {
                    successDialog.setVisible(true);
                    successDialog.showAnimation();
                }
            }

            @Override
            public void next() {
                nextTouched();
            }

            @Override
            public CommonAnimation coinGot(int coin) {
                return GameScreen.this.coinGot(coin,true);
            }

            @Override
            public void limitGoodsGot(int id) {
                GameScreen.this.limitGoods(id,true);
            }

            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                playVideoReward(action,pendingValue);
            }
        });
    }
    protected void showBonusBoxDialog(){
        setPendingAction(PendingAction.NONE);
        FlurryUtils.Ads_Reward("Box");
        SuccessBonusBoxDialog dialog = (SuccessBonusBoxDialog)showDialog("ui/dialogs/box_bonus.json", SuccessBonusBoxDialog.class);
        successBonusBaseDialog = dialog;
        if(successDialog!=null) {
            successDialog.setVisible(false);
        }
        dialog.setSuccessBonusBaseDialogListener(new SuccessBonusBaseDialog.successBonusBaseDialogListener() {
            @Override
            public void showSuccessDialog() {
                if(successDialog!=null) {
                    successDialog.setVisible(true);
                    successDialog.showAnimation();
                }
            }

            @Override
            public void next() {
                nextTouched();
            }

            @Override
            public CommonAnimation coinGot(int coin) {
                return GameScreen.this.coinGot(coin,true);
            }

            @Override
            public void limitGoodsGot(int id) {
                GameScreen.this.limitGoods(id,true);
            }

            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                playVideoReward(action,pendingValue);
            }
        });
    }



    @Override
    protected boolean back() {
        Log.log("GameScreen", "back()");
        if (dialogs.size() > 0) {
            BaseDialog dialog = dialogs.peek();
            if (dialog != null) {
                if (dialog instanceof FailureDialog || dialog instanceof SuccessDialog || dialog instanceof SuccessBonusBaseDialog) {
                    return false;
                } else {
                    dialog.close();
                    return false;
                }
            }
        } else {
            Log.log("GameScreen", "back()--->pause");
            pause();
        }
        return true;
    }

    @Override
    public void end(Class screen) {
        super.end(screen);
        outTime = 0.8F;
        layer.remove();
        stage.addActor(circleLayer);
        circleLayer.setOutTime(outTime-0.2F);
        circleLayer.close(new CircleLayer.LayerListener() {
            @Override
            public void end() {
            //    stateTime = outTime+2;
            }
        });
    }

    @Override
    public void dispose() {
        super.dispose();
        gamePanel.dispose();
        gamePanel = null;
        uiPanel = null;
    }

    public Vector2 screenToStageCoordinates(float x, float y) {
        Vector2 temp = stage.screenToStageCoordinates(new Vector2(x, Gdx.graphics.getBackBufferHeight() - y));
        return temp;
    }
    public enum GameType{
        NORMAL,HAPPY_GLASS
    }
}