package com.zyb.loveball;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.UnlockBallDialog;
import com.zyb.loveball.listeners.FullScreenListener;
import com.zyb.loveball.objects.NoviceCourse;
import com.zyb.loveball.objects.ProgressBar;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.SoundButtonListener;

public class GameUIPanel extends UIPanel {
    private ProgressBar progressBar;
    int star = 3;
    private CommonAnimation[] starAnimation;
    private CommonAnimation handAnimation;
//    private boolean inkProgressCourse;

    boolean handTip;
    Group hintVideo;
    Actor hintFree;
    Group hintCoin;
    Group hintEnable;
    Actor hintDisable;
    Actor hintVideoGray;
    Group hintBtn;
    private HintState hintState = HintState.none;

    public GameUIPanel(Group scene, GameScreen screen) {
        super(scene, screen);
        this.root = scene.findActor("normal");
        root.setVisible(true);
        progressBar = new ProgressBar(root.findActor("progress_bar"), (Label) root.findActor("has_remain_percent"));
        starAnimation = new CommonAnimation[3];
        for (int i = 0; i < starAnimation.length; i++) {
            starAnimation[i] = new CommonAnimation("animations/star.skel", 1);
            Actor temp = root.findActor("star_small_empty" + i);
            starAnimation[i].setPosition(temp.getX() + 10, temp.getY() + 10);
            root.addActorAfter(temp, starAnimation[i]);
        }
        init();
    }

    private void init() {
        if (screen.getClass() != screen.getGame().getPreScreen()) {
            GameInfo.reset();
        }
        hintBtn = root.findActor("btn_hint");
        hintVideo = root.findActor("video");
        hintVideoGray = root.findActor("video_gray");
        hintCoin = root.findActor("coin");
        hintEnable = root.findActor("hint_enable");
        hintDisable = root.findActor("hint_disable");
        hintFree = root.findActor("free");
        ((Label) hintCoin.findActor("hint_value")).setText(GameInfo.levelData.getHintPrice() + "");
        Group bottom = root.findActor("bottom");
        if (bottom != null) {
            bottom.setY(800 - HelloZombieGame.getViewport().getWorldHeight());
        }
        Group center = root.findActor("center");
        if (center != null) {
            center.setY((800 - HelloZombieGame.getViewport().getWorldHeight())/2);
        }
        initButtons();
        showSkip(false);

    }

    @Override
    public void setGamePanel(GamePanel gamePanel) {
        super.setGamePanel(gamePanel);
        if((GameInfo.chapter > 1 || GameInfo.id >= 3)&&
                (!Configuration.settingData.isInkProgressCourse()||!Configuration.settingData.isHintCourse()||!Configuration.settingData.isRetryCourse()||!Configuration.settingData.isNetCourse())){
            System.out.println("new course");
            noviceCourse = new NoviceCourse(screen,root);
            noviceCourse.setPosition(0,(Configuration.height-800)+700);
            if(!Configuration.poor) {
                ((Group) root.findActor("bottom")).addActorBefore(magnifierUtil, noviceCourse);
            }else {
                ((Group) root.findActor("bottom")).addActor(noviceCourse);
            }
        }
        if (GameInfo.course) {
            initPlayMethodCourse();
            hideButton();
        }
        checkHint();
    }

    private void hideButton(){

        root.findActor("btn_hint").setVisible(false);
        root.findActor("btn_pause").setVisible(false);
    }

    private void initPlayMethodCourse() {
//        Vector3 vector3 = gamePanel.getHintLineScreenPosition();
        Vector3 vector3 = gamePanel.getZombieScreenPosition();
        Group temp = screen.parseScene("ui/objects/course_object.json");
        Vector2 zomPosition = screen.screenToStageCoordinates(vector3.x, vector3.y);
        temp.setPosition(zomPosition.x+20,zomPosition.y+50);
        temp.setTouchable(Touchable.disabled);
        temp.setName("course_object");
        ((Group) root.findActor("bottom")).addActor(temp);

    }

    private void hideCourse(){
        Actor actor = root.findActor("course_object");
        if(actor == null) return;
        actor.remove();
//        actor = root.findActor("course_label");
//        if(actor!=null) actor.remove();

    }

    @Override
    public void showHand(boolean show) {
        if (show == handTip) return;
        if (show) {
            if (GameInfo.levelData.isShowHint()) return;
            if (hintState == HintState.none) return;
            if (handAnimation == null) {
                initHandTipAni();
            }
            handAnimation.setVisible(true);
            handAnimation.getState().setAnimation(0, "1", false);
            handAnimation.getState().addAnimation(0, "2", true, 0.5f);
            handTip = true;
        } else {
            if (handAnimation != null)
                handAnimation.getState().setAnimation(0, "3", false);
            handTip = false;
        }
    }

    private void initHandTipAni() {
        handAnimation = new CommonAnimation("animations/dianji.skel", 0.5f);
        root.addActor(handAnimation);
        Actor hint = root.findActor("btn_hint");
        float x = hint.getX();
        float y = hint.getY();
        handAnimation.setPosition(x, y);
        handAnimation.setxOffset(hint.getWidth() / 2);
        handAnimation.setyOffset(hint.getHeight() / 2);
        handAnimation.setVisible(false);
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        hintBtn.addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                hint();
                if(noviceCourse!=null){
                    if(noviceCourse.courseState == NoviceCourse.CourseState.hint_course){
//                        noviceCourse.courseState = NoviceCourse.CourseState.none;
//                        noviceCourse.update();
                        noviceCourse.setNone();
                    }
                }
            }
        });
        root.findActor("btn_skip", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                gamePanel.scene.getZombie().boom(true,1);
                gamePanel.end(true, 1.5F);
                showSkip(false);
                FlurryUtils.level_Skip(GameInfo.chapter, GameInfo.id);
            }
        });
        Label levelLabel = root.findActor("level_label");
        if(GameInfo.type == 0) {
            levelLabel.setText("Level " + GameInfo.chapter + "-" + GameInfo.id);
        }else {
            levelLabel.setText("");
        }
        if (GameInfo.levelData.isBanner()) {
            levelLabel.setY(levelLabel.getY() + 60);
            root.findActor("btn_skip").setY(root.findActor("btn_skip").getY()+70);
        }
        CommonAnimation animation = new CommonAnimation("animations/tishi.skel", 0.5f);
        animation.setPosition(hintEnable.getWidth() / 2 - 3F, hintEnable.getHeight() / 2 + 0.3F);
        animation.setName("hint_animation");
        hintEnable.addActor(animation);
        hintAniReset();
    }

    public void showSkip(boolean show) {
        root.findActor("btn_skip").setVisible(show);
    }

    public void screenShot() {
        root.addAction(Actions.sequence(Actions.delay(0.05F), Actions.run(new Runnable() {
            @Override
            public void run() {
                MScreenUtil.screenShot(screen.getGame(), gamePanel, null, Constant.SCREEN_SHOT_W_BIG, Constant.SCREEN_SHOT_H_BIG, true);
            }
        })));

    }

    private void hint() {
        if (GameInfo.hint) return;
//        if (hintState == HintState.coin || hintState == HintState.free) {
        if (hintState == HintState.free) {
            Configuration.settingData.cost(GameInfo.levelData.getHintPrice());
            gamePanel.hint();
            FlurryUtils.level_HintCoin(GameInfo.chapter, GameInfo.id);
        } else if (hintState == HintState.video) {
            if(Configuration.settingData.isFirstWatchVideo()){
                Configuration.settingData.setFirstWatchVideo(false);
                screen.showHintConfirmDialog();
            }else {
                screen.setPendingAction(BaseScreen.PendingAction.HINT);
                HelloZombieGame.launcherListener.playUnityAds();
                FlurryUtils.level_HintAD(GameInfo.chapter, GameInfo.id);
            }
        } else {
            screen.showMessage("The video ad is not ready!");
        }
        updateHintBtn();
        showHand(false);
        hintAniReset();
    }

    private void checkHint() {
        updateHintBtn();
        if (GameInfo.levelData.isShowHint()) return;
        if (GameInfo.hint) return;
        if (GameInfo.failureTimes == 0) return;
        if (hintState == HintState.none) return;
        CommonAnimation animation = root.findActor("hint_animation");
        Group hintGroup = root.findActor("btn_hint");
        animation.getState().setAnimation(0, "animation", true);
        hintGroup.addAction(Actions.forever(Actions.sequence(Actions.scaleTo(1.05F, 1.05F, 0.4F), Actions.scaleTo(1, 1, 0.4F))));
    }

    private void hintAniReset() {
        CommonAnimation animation = root.findActor("hint_animation");
        animation.getState().clearTracks();
        animation.getSkeleton().setToSetupPose();
        animation.getState().setAnimation(0, "animation2", true);
        root.findActor("btn_hint").getActions().clear();
    }


    @Override
    public void update() {
        if (gamePanel == null) return;
        float progress = gamePanel.inkLeftPercent();
        progressBar.update(progress);
        if (progress < 0) progress = 0;
        if (progress < 0.7 && star == 3) {
            star--;
            starAnimation[2].getState().setAnimation(0, "animation", false);
        }
        if (progress < 0.4 && star == 2) {
            star--;
            starAnimation[1].getState().setAnimation(0, "animation", false);
        }
        GameInfo.star = star;
        updateHintBtn();
    }

    public void end() {
        showHand(false);
        hintAniReset();
    }

    private synchronized void updateHintBtn() {
        if (gamePanel.getState() == GamePanel.GameState.end) return;
        if (GameInfo.hint) {
            hintDisable.setVisible(true);
            hintEnable.setVisible(false);
            hintBtn.setTouchable(Touchable.disabled);
        } else {
            hintDisable.setVisible(false);
            hintEnable.setVisible(true);
            hintBtn.setTouchable(Touchable.enabled);
        }
        hintEnable.findActor("hint_animation").setVisible(true);
        if (GameInfo.levelData.getHintPrice() == 0) {
            hintVideo.setVisible(false);
            hintCoin.setVisible(false);
            hintFree.setVisible(true);
            hintState = HintState.free;
        }
//        else if (Configuration.settingData.getCoin() >= GameInfo.levelData.getHintPrice()) {
//            hintVideo.setVisible(false);
//            hintFree.setVisible(false);
//            hintCoin.setVisible(true);
//            hintState = HintState.coin;
//        }
        else {
            hintVideo.setVisible(true);
            hintCoin.setVisible(false);
            hintFree.setVisible(false);
            Configuration.videoAdReady = HelloZombieGame.launcherListener.isVideoAdReady();
            if (Configuration.videoAdReady) {
                hintVideoGray.setVisible(false);
                hintVideo.setTouchable(Touchable.enabled);
                hintState = HintState.video;

            } else {
                hintVideoGray.setVisible(true);
                hintVideo.setTouchable(Touchable.childrenOnly);
                hintState = HintState.none;
                hintEnable.findActor("hint_animation").setVisible(false);
            }
        }
    }

    public void success() {
//        screen.closeBanner();
//        screen.showBanner = false;
        if(!Configuration.effectiveBoot){

            Configuration.effectiveBoot = true;
            Configuration.settingData.addEffectiveBootTimesPerDay();
        }


//        if(GameInfo.levelData.isRateLevel() && !Configuration.settingData.isUseHint(GameInfo.levelFileName) && GameInfo.star == 3){
        if(GameInfo.levelData.isRateLevel() && !gamePanel.hint.isUseHint() && GameInfo.star == 3){
            if(GameInfo.levelFileName<=1000) {
                Configuration.settingData.setHardLevel3StarNoHint(GameInfo.levelFileName);
            }
        }
        int checkId = checkStarUnlock();
        if (checkId == -1) {
            screen.success();
        } else {
            UnlockBallDialog dialog = (UnlockBallDialog) screen.showDialog("ui/dialogs/star_unlock_time_limited_skin.json", UnlockBallDialog.class);
            dialog.setData(checkId);
            dialog.setUnlockBallDialogListener(new UnlockBallDialog.UnlockBallDialogListener() {
                @Override
                public void success() {
                    screen.success();
                }
            });
        }
    }

    private int checkStarUnlock() {
        Configuration.settingData.setStar(GameInfo.chapter, GameInfo.id, GameInfo.star, GameInfo.type);
        int totalStar = Configuration.settingData.getStarTotal();
        int ballNum = Assets.instance.getGoodsNum(1);
        for (int i = 1; i <= ballNum; i++) {
            if (!Configuration.settingData.isGoodUnlocked(i, 1)) {
                GoodData data = Assets.instance.getGoodData(i, 1);
                if (data.getCondition() == 1) {
                    if (totalStar >= data.getCoin()) {
                        return i;
                    }
                }
            }
        }
        return -1;
    }
    @Override
    public void failure(boolean isRetryBtn) {
        super.failure(isRetryBtn);
        Configuration.retry_times_full_screen++;
        boolean showAd = BaseScreen.showFullScreenAd(0, new FullScreenListener() {
            @Override
            public void adClosed() {
                screen.end(GameScreen.class);
            }
        });
        if(!showAd) {
            screen.end(GameScreen.class);
        }
        GameInfo.failureTimes++;
        FlurryUtils.level_Fail(GameInfo.chapter, GameInfo.id);
        if (Configuration.settingData.isFirstPlay()) {
            FlurryUtils.level_FirstRestart(GameInfo.chapter, GameInfo.id);
        }
//        if(isRetryBtn) {
//            screen.showFullScreenAd(0);
//        }else {
//            screen.showFullScreenAd(0.2f);
//        }
    }

    @Override
    public void updateWorldPosition(Vector2 position) {
        super.updateWorldPosition(position);
//        if(inkProgressCourse){
//            Actor courseGroup = root.findActor("ink_progress_course");
//            if(courseGroup!=null){
////                courseGroup.setVisible(false);
////                courseGroup.remove();
//                inkProgressCourse = false;
//            }
//
//        }
        if(noviceCourse!=null){
            if(noviceCourse.courseState == NoviceCourse.CourseState.ink_progress_course){
//                noviceCourse.courseState = NoviceCourse.CourseState.none;
//                noviceCourse.update();
                noviceCourse.setNone();
            }
        }
        if(GameInfo.course) {

            hideCourse();
        }
    }

    public enum HintState {
//        none, coin, video, free
        none, video, free

    }
}
