package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.objects.MagnifierUtil;
import com.zyb.loveball.objects.NoviceCourse;

public class UIPanel {
    Group root;
    private Label debugLabel;
    protected GameScreen screen;
    GamePanel gamePanel;
    protected boolean countdown;
    protected int countdownTime;
    MagnifierUtil magnifierUtil;
    public NoviceCourse noviceCourse;

    public UIPanel(Group scene, GameScreen screen) {
        this.screen = screen;
        this.gamePanel = screen.gamePanel;
        Actor temp = scene.findActor("normal");
        temp.setTouchable(Touchable.childrenOnly);
        temp.setVisible(false);
        temp = scene.findActor("challenge");
        temp.setTouchable(Touchable.childrenOnly);
        temp.setVisible(false);
        if (Configuration.debug)
            initDebug();
    }


    public void initAnimation() {
    }


    public void showHand(boolean show) {

    }

    public void countdown(boolean show,int time){
        if(countdown == show) return;
        countdown = show;
        final Group countdownGroup = root.findActor("time_countdown");
        if(countdown){
            countdownTime = time;
            countdownGroup.setVisible(true);
            countdownGroup.addAction((Actions.forever(Actions.sequence(Actions.run(new Runnable() {
                @Override
                public void run() {
                    if(countdownTime>=0) {
                        ((Label) countdownGroup.findActor("time_label")).setText("" + countdownTime);
                    }
                    if(countdownTime < 0){
                        gamePanel.end(false,0);
                    }
                    if(gamePanel.state !=GamePanel.GameState.pause) {
                        countdownTime -= 1;
                    }
                }
            }),Actions.delay(1)))));
        }else{
            countdownGroup.setVisible(false);
            countdownGroup.getActions().clear();
        }
    }
//    public void showHintCourse(){
//    }
//    public void showRetryCourse(){
//    }
//    public void setCourseVisible(int type,boolean visible){
//    }

    public void hideButtons(){
        if(screen!=null){
            screen.getScene().findActor("top").setVisible(false);
            screen.getScene().findActor("bottom").setVisible(false);
        }
    }

    public void setGamePanel(GamePanel gamePanel) {
        this.gamePanel = gamePanel;
        magnifierUtil = new MagnifierUtil(GameInfo.levelData.getMagnifier(), gamePanel);
        if(!Configuration.poor) {
            ((Group) root.findActor("bottom")).addActor(magnifierUtil);
        }
    }

    protected void initButtons() {
        initAnimation();
    }

    private void initDebug() {
        Label.LabelStyle style = new Label.LabelStyle();
        style.font = Assets.instance.fonts[0];
        style.fontColor = new Color(0, 0, 1, 1);
        debugLabel = new Label("FPS:" + Gdx.graphics.getFramesPerSecond(), style) {
            @Override
            public void act(float delta) {
                super.act(delta);
                float hintLength = 0;
                if (gamePanel.getPaintPanel().hint != null) {
                    hintLength = gamePanel.getPaintPanel().hint.getHintLength();
                }
                debugLabel.setText("FPS:    " + Gdx.graphics.getFramesPerSecond()
                        + "\n Render Calls:  " + ((PolygonSpriteBatch) HelloZombieGame.getBatch()).renderCalls
                        + "\n Hint Length:  " + hintLength
                        + "\n neiCunJava:  " + Gdx.app.getJavaHeap()
                        + "   neiCunNative:  " + Gdx.app.getNativeHeap());
            }
        };
        debugLabel.setFontScale(0.3F, 0.3F);
        screen.getScene().addActor(debugLabel);
    }

    public void showSkip(boolean show) {
    }


    public void screenShot() {
    }


    public void update() {

    }

    public void success() {

    }

    public void updateWorldPosition(Vector2 position) {
        if (Configuration.poor) return;
        magnifierUtil.update(position);

    }

    public void magnifier(boolean magnifier) {
        if (Configuration.poor) return;
        magnifierUtil.setMagnifier(magnifier);
    }

    public Group getRoot() {
        return root;
    }

    public void failure(boolean isRetryBtn) {

    }

    public void end(){

    }
}
