package com.zyb.loveball.assets;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.resolvers.LocalFileHandleResolver;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.loader.ChapterDataLoader;
import com.zyb.loveball.loader.CocosUILoader;
import com.zyb.loveball.loader.GoodDataLoader;
import com.zyb.loveball.loader.LevelDataLoader;
import com.zyb.loveball.loader.texture.MiniTextureAtlasLoader;
import com.zyb.loveball.loader.texture.MiniTextureLoader;
import com.zyb.loveball.utils.CsvReader;
import com.zyb.loveball.utils.Log;

import org.freyja.libgdx.cocostudio.ui.model.CCExport;

import java.nio.charset.Charset;

public class Assets {
    public TextureAtlas game;
    public TextureAtlas ui;
    public TextureAtlas animation;
    public AssetManager assetManager;
    public AssetManager assetManagerLocal;
    public static String[] FONT_NAME = {"comic.fnt", "comicbd.fnt"};
    public BitmapFont[] fonts = new BitmapFont[FONT_NAME.length];
    public static Assets instance;
    public LevelDataLoader.LevelsData levelsData;
    public ChapterDataLoader.ChaptersData chaptersData;
    private GoodDataLoader.GoodsData goodsData;

    public String prefix = "image/";

    private Assets() {
        if (Gdx.app.getType() != Application.ApplicationType.Android) {
            prefix = "desktop/";
        }
        assetManager = new AssetManager();
        assetManager.setLoader(ChapterDataLoader.ChaptersData.class, new ChapterDataLoader(assetManager.getFileHandleResolver()));
        assetManager.setLoader(LevelDataLoader.LevelsData.class, new LevelDataLoader(assetManager.getFileHandleResolver()));
        assetManager.setLoader(CCExport.class, new CocosUILoader(assetManager.getFileHandleResolver()));
        assetManager.setLoader(GoodDataLoader.GoodsData.class, new GoodDataLoader(assetManager.getFileHandleResolver()));
        assetManagerLocal = new AssetManager(new LocalFileHandleResolver());
    }

    public static void prepare() {
        instance = new Assets();
        instance.init();
    }

    private void init() {
        if (Configuration.poor) {
            assetManager.setLoader(Texture.class, new MiniTextureLoader(assetManager.getFileHandleResolver()));
            assetManagerLocal.setLoader(Texture.class, new MiniTextureLoader(assetManagerLocal.getFileHandleResolver()));
            assetManager.setLoader(TextureAtlas.class, new MiniTextureAtlasLoader(assetManager.getFileHandleResolver()));
            assetManagerLocal.setLoader(TextureAtlas.class, new MiniTextureAtlasLoader(assetManagerLocal.getFileHandleResolver()));
        }
    }

    public void load() {
        assetManager.load(prefix + "game_.atlas", TextureAtlas.class);
        assetManager.load(prefix + "ui_.atlas", TextureAtlas.class);
        assetManager.load(prefix + "animation_.atlas", TextureAtlas.class);
        loadScenes();
        loadData();
        loadChaptersAtlas();
        for (int i = 0; i < fonts.length; i++) {
            assetManager.load("font/" + FONT_NAME[i], BitmapFont.class);
        }
//        loadLocalLevelPics();
    }

    public void loadFinish() {
        game = assetManager.get(prefix + "game_.atlas", TextureAtlas.class);
        ui = assetManager.get(prefix + "ui_.atlas", TextureAtlas.class);
        animation = assetManager.get(prefix + "animation_.atlas", TextureAtlas.class);
        for (int i = 0; i < fonts.length; i++) {
            fonts[i] = assetManager.get("font/" + FONT_NAME[i]);
        }
        levelsData = Assets.instance.assetManager.get("data/normal_stage.csv", LevelDataLoader.LevelsData.class);
        chaptersData = Assets.instance.assetManager.get("data/chapter.csv", ChapterDataLoader.ChaptersData.class);
        goodsData = Assets.instance.assetManager.get("data/skin.csv", GoodDataLoader.GoodsData.class);
    }

    private void loadData() {
        loadWorldConfig();
        assetManager.load("data/chapter.csv", ChapterDataLoader.ChaptersData.class);
        assetManager.load("data/normal_stage.csv", LevelDataLoader.LevelsData.class);
        assetManager.load("data/skin.csv", GoodDataLoader.GoodsData.class);
    }

    private void loadScenes() {
        assetManager.load("ui/game.json", CCExport.class);
        assetManager.load("ui/chapter.json", CCExport.class);
        assetManager.load("ui/level.json", CCExport.class);
        assetManager.load("ui/menu.json", CCExport.class);
        assetManager.load("ui/challenge.json", CCExport.class);
        assetManager.load("ui/store.json", CCExport.class);
    }

    private void loadWorldConfig() {
        CsvReader reader = new CsvReader(Gdx.files.internal("data/world_config.csv").read(), Charset.defaultCharset());
        try {
            reader.skipLine();
            reader.readHeaders();
            while (reader.readRecord()) {
                String temp = null;
                temp = reader.get("id");
                if (temp == null || temp.equals("")) continue;
                switch (temp) {
                    case "ball":
                        parseAttribute(reader, Constant.ball);
                        break;
                    case "zombie":
                        parseAttribute(reader, Constant.zombie);
                        break;
                    case "line":
                        parseAttribute(reader, Constant.line);
                        break;
                    case "ground":
                        parseAttribute(reader, Constant.ground);
                        break;
                    case "dynamic_object":
                        parseAttribute(reader, Constant.dynamic_object);
                        break;
                    case "seesaw":
                        parseAttribute(reader, Constant.seesaw);
                        break;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GoodData getGoodData(int id, int type) {
        if (type == 1) {
            id += 100;
        }else if(type == 2){
            id +=1000;
        }
        return goodsData.get(id);
    }

    public int getGoodsNum(int type) {
        if (type == 0) {
            return goodsData.penNum;
        } else if (type == 1) {
            return goodsData.ballNum;
        } else if(type == 2){
            return goodsData.zombieNum;
        } else {
            return 0;
        }
    }

    private void parseAttribute(CsvReader reader, Constant.BaseAttribute attribute) {
        try {
            String tempStr = reader.get("density");
            if (tempStr != null && !tempStr.equals("")) {
                attribute.density = Float.parseFloat(tempStr);
            }
            tempStr = reader.get("restitution");
            if (tempStr != null && !tempStr.equals("")) {
                attribute.restitution = Float.parseFloat(tempStr);
            }
            tempStr = reader.get("friction");
            if (tempStr != null && !tempStr.equals("")) {
                attribute.friction = Float.parseFloat(tempStr);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void loadChaptersAtlas() {
        if(assetManager.isLoaded(prefix + "chapters_.atlas")) return;
        assetManager.load(prefix + "chapters_.atlas", TextureAtlas.class);
    }
    public void unloadChaptersAtlas() {
        if(assetManager.isLoaded(prefix + "chapters_.atlas")) {
            assetManager.unload(prefix + "chapters_.atlas");
        }
    }

    public void loadLevelAtlas(int chapter) {
//        if(assetManager.isLoaded(prefix + "chapter_" + chapter + ".atlas")) return;
        assetManager.load(prefix + "chapter_" + chapter + "_.atlas", TextureAtlas.class);
    }
    public void unloadLevelAtlas(int chapter) {
        if(assetManager.isLoaded(prefix + "chapter_" + chapter + "_.atlas")) {
            assetManager.unload(prefix + "chapter_" + chapter + "_.atlas");
        }
    }



    public static void dispose() {
        Log.log("Assets", "dispose");
//        instance.assetManagerLocal.dispose();
//        instance.assetManager.dispose();
//        instance.assetManager = null;
//        instance.assetManagerLocal = null;
//        instance = null;

    }


}
