package com.zyb.loveball.assets;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Base64Coder;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.utils.Log;

import java.text.DecimalFormat;
import java.util.Date;

public class Configuration {
    public static SettingData settingData;
    private static Preferences preferences;
    public static boolean unlock = false;
    public static boolean box2dDebug = false;
    public static boolean debug = false;
    public static boolean videoAdReady = false;
    public static float full_screen_time;
    public static int retry_times_full_screen;
    public static boolean poor = false;
    public static String device_name = "";
    public static int width;
    public static int height;
    public static int screen_width = 480;
    public static int screen_height = 800;
    public static int numSamples = 4;
    public static int availableMem;
    public static int APILevel = 19;
    public static float banner_time;
    public static BannerState banner_state = BannerState.none;
    public static int[] bannerVideoInt = null;
    public static boolean effectiveBoot;
    public static boolean effectiveBootRate;
    public static int versionCode;

    public static DecimalFormat numType = new DecimalFormat("#,###");

    public static void init_device() {
        if (screen_width < 480 ||
                screen_height < 800 ||
                (screen_width * screen_height < 800 * 480) ||
                "DROID X2".equals(device_name) ||
                "MT25i".equals(device_name)) {
            poor = true;
        } else {
            poor = false;
        }
    }

    public static void prepare(){
        if(versionCode == 0){
            versionCode = 5;
        }
        preferences = Gdx.app.getPreferences("game_data");
        if(Gdx.gl20 == null){
            poor = true;
        }
        reset();
        loadSettingData();
    }

    public static void reset() {
        banner_time = 0;
        banner_state = BannerState.none;
        resetFullScreenTime();
        effectiveBootRate = false;
    }
    public static void resetFullScreenTime(){
        full_screen_time = 0;
        retry_times_full_screen = 0;
    }

    private static void loadSettingData() {
        String data = preferences.getString("setting_data", "");
        if (!"".equals(data)) {
            try {
                String base64Data = Base64Coder.decodeString(data);
                settingData = HelloZombieGame.getJson().fromJson(SettingData.class, base64Data);
                if (settingData.isFirstPlay()) {
                    settingData.setFirstPlay(false);
                }
            } catch (Exception e) {
                e.printStackTrace();
                try {
                    settingData = HelloZombieGame.getJson().fromJson(SettingData.class, data);
                    if (settingData.isFirstPlay()) {
                        settingData.setFirstPlay(false);
                    }
                }catch (Exception e2){
                    e2.printStackTrace();
                }
            }
        }
        if (settingData == null) {
            settingData = new SettingData();
        }
        initSettingData();
    }

    private static void initSettingData(){
        if(settingData.getInstall_time() == -1){
            settingData.setInstall_time(System.currentTimeMillis());
        }
        long lastGameTime = settingData.getGame_time();
        long currentTime = System.currentTimeMillis();
        settingData.setGame_time(currentTime);
        if(lastGameTime!=-1){
            Date lastDate = new Date(lastGameTime);
            Date current = new Date(currentTime);
            if(lastDate.getDate() != current.getDate()){
                settingData.resetEffectiveBootTimesPerDay();
            }
        }
        checkLevelData();
        settingData.setVersionCode(versionCode);
        checkSettingDateZombie();
    }

    private static void checkLevelData(){
        Log.log("Configuration","version5");
        int unlockChapter = settingData.getUnlockChapter();
        int unlockLevel = settingData.getUnlockLevel();
        boolean [][] levels = settingData.getLevels();
        if(levels == null || !levels[unlockChapter][unlockLevel]) {
            for (int i = 1; i <= unlockChapter; i++) {
                for (int j = 1; j <= 10; j++) {
                    settingData.unlockLevel(i, j,0);
                    if (i == unlockChapter && j == unlockLevel) {
                        break;
                    }
                }
            }
        }
    }
    private static void checkSettingDateZombie(){
        if(!settingData.isGoodUnlocked(1,2)){
            settingData.unlockGood(1,2);
        }
    }

    public static void saveData() {
        if (settingData == null) return;
        String str = settingData.toString();
        if(str!=null) {
            String base64Str = Base64Coder.encodeString(str);
            preferences.putString("setting_data", base64Str);
            preferences.flush();
        }
    }

    public static void resetData() {
        settingData = new SettingData();
        saveData();
    }

    public static enum BannerState{
        none,ad,video
    }

    public static boolean isShowRate(){
        if(settingData.isRate()) return false;

        // 24小时且31关以后,第一次三星通关
        if(!settingData.isRateShow(0)) {
            float timeOffset = (settingData.getGame_time() - settingData.getInstall_time())/(1000F*60*60);
            int passedLevel = (GameInfo.chapter - 1) * 10 + GameInfo.id;
            if(GameInfo.star==3 && passedLevel >=31 && timeOffset<=24 && GameInfo.type == 0){
                settingData.rateShow(0);
                return true;
            }
        }

        //购买第四个炸弹皮肤或者第四只笔以后，第一次三星通关
        if(!settingData.isRateShow(1) && isGood4Unlocked() && GameInfo.star == 3){
            settingData.rateShow(1);
            return true;
        }

        //一天内有效重启次数(启动游戏，并且成功通关一关以上)≥5，在玩家顺利三星通关之后
        if(!settingData.isRateShow(2) && !effectiveBootRate && GameInfo.star == 3 && settingData.getEffectiveBootTimesPerDay()>=5){
            effectiveBootRate = true;
            settingData.rateShow(2);
            return true;
        }

        //N个指定关卡(normal_stage.xlsx)里有5个不用提示，手动三星通关。

        if(!settingData.isRateShow(3) && settingData.getHardLevelPassNum()>=5 && GameInfo.star == 3){
            settingData.rateShow(3);
            return true;
        }

        return false;
    }

    //等于-1为金币不够，不满足条件
    public static int checkDiscountBallId(float discount) {
        int goodNum = Assets.instance.getGoodsNum(1);
        int id = -1;
        for (int i = 1; i <= goodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            GoodData goodData = Assets.instance.getGoodData(i, 1);
            if (goodData.getCondition() == 1) continue;
            if (goodData.getCoin()*discount <= Configuration.settingData.getCoin()){
                id = i;
                break;
            }

        }
        return id;
    }
    //笔，球-100，僵尸-1000
    public static int checkLimitId() {
        int penGoodNum = Assets.instance.getGoodsNum(0);
        int ballGoodNum = Assets.instance.getGoodsNum(1);
        int zombieGoodNum = Assets.instance.getGoodsNum(2);
        long currentTime = System.currentTimeMillis();
        Array<Integer> temp = new Array<>();
        for (int i = 1; i <= penGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 0)) continue;
            if (currentTime < Configuration.settingData.getPenDeadlineTime(i)) continue;
            temp.add(i);
        }
        for (int i = 1; i <= ballGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            if (currentTime < Configuration.settingData.getBallDeadlineTime(i)) continue;
            temp.add(i+100);
        }
        for (int i = 1; i <= zombieGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 2)) continue;
            if (currentTime < Configuration.settingData.getZombieDeadlineTime(i)) continue;
            temp.add(i+1000);
        }
        int id = -1;
        if(temp.size>0){
            id = temp.random();
        }
        return id;
    }
    public static int checkLimitPenId() {
        int penGoodNum = Assets.instance.getGoodsNum(0);
        long currentTime = System.currentTimeMillis();
        Array<Integer> temp = new Array<>();
        for (int i = 1; i <= penGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 0)) continue;
            if (currentTime < Configuration.settingData.getPenDeadlineTime(i)) continue;
            temp.add(i);
        }
        int id = -1;
        if(temp.size>0){
            id = temp.random();
        }
        return id;
    }
    public static int checkLimitBallId() {
        int ballGoodNum = Assets.instance.getGoodsNum(1);
        long currentTime = System.currentTimeMillis();
        Array<Integer> temp = new Array<>();
        for (int i = 1; i <= ballGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            if (currentTime < Configuration.settingData.getBallDeadlineTime(i)) continue;
            temp.add(i+100);
        }
        int id = -1;
        if(temp.size>0){
            id = temp.random();
        }
        return id;
    }
    public static int checkLimitZombieId() {
        int zombieGoodNum = Assets.instance.getGoodsNum(2);
        long currentTime = System.currentTimeMillis();
        Array<Integer> temp = new Array<>();
        for (int i = 1; i <= zombieGoodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 2)) continue;
            if (currentTime < Configuration.settingData.getZombieDeadlineTime(i)) continue;
            temp.add(i+1000);
        }
        int id = -1;
        if(temp.size>0){
            id = temp.random();
        }
        return id;
    }


    private static boolean isGood4Unlocked(){
        int ballNum = Assets.instance.getGoodsNum(1);
        for(int i = 4;i<=ballNum;i++){
            if(settingData.isGoodUnlocked(i,1)){
                return true;
            }
        }
        int penNum = Assets.instance.getGoodsNum(0);
        for(int i = 4;i<=penNum;i++){
            if(settingData.isGoodUnlocked(i,0)){
                return true;
            }
        }
        return false;
    }
}
