package com.zyb.loveball.dialogs;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.objects.Box;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.LClickListener;

/**
 * Created by Doodle on 2019/3/25.
 */

public class SuccessBonusBoxDialog extends SuccessBonusBaseDialog{
    final int BOX_NUM = 4;
    Box[] boxes = new Box[4];
    BoxState boxState = BoxState.none;
    @Override
    public void load(Group group) {
        super.load(group);
        init();
    }

    @Override
    public void init() {
        super.init();
        initLimitGoods(-1);
        initAnimation();
    }


    private void initAnimation() {
        for(int i = 0; i< BOX_NUM; i++){
            boxes[i] = new Box((Group) group.findActor("box"+(i+1), Touchable.enabled),i+1);
            boxes[i].setBoxListener(new Box.BoxListener() {
                @Override
                public boolean changeDialogState(BoxState fromState, BoxState targetState) {
                    return changeState(fromState,targetState);
                }

                @Override
                public void updateDialogCoin() {
                    updateCoin();
                }
            });
            float width = boxes[i].getGroup().getWidth();
            float height = boxes[i].getGroup().getHeight();
            boxes[i].getGroup().setOrigin(width/2,height/2);
            boxes[i].addShuffleAction(BOX_NUM);
        }
    }
    public boolean changeState(BoxState fromState, BoxState targetState){
        if(boxState == fromState){
            boxState = targetState;
            update();
            return true;
        }
        return false;
    }
    private int openInt = 0;
    private int[] openCost = {0,100,200,300,0};
    @Override
    protected void initButtons() {
        super.initButtons();
        initBoxButton();
        group.findActor("btn_next",Touchable.enabled).addListener(new ButtonListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                GameScreen.canShowFullAds = false;

                    successBonusBaseDialogListener.next();

            }
        });
        group.findActor("btn_open_all",Touchable.enabled).addListener(new ButtonListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                successBonusBaseDialogListener.playVideo(BaseScreen.PendingAction.SECOND_BONUS,3);
//                flopAll();
            }
        });
    }

    private void initBoxButton() {
        for(int i = 0; i< BOX_NUM; i++){
            final Group boxGroup = group.findActor("box"+(i+1), Touchable.enabled);
            final int boxInt = i;
            boxGroup.addListener(new LClickListener(){
                @Override
                public void touchDownEffect() {
                    super.touchDownEffect();
                    if(boxGroup.findActor("back").isVisible()){
                        if(boxState == BoxState.waitForChoose){
                            if(Configuration.settingData.cost(openCost[openInt])) {
                                updateCoin();
                                flop(boxInt,false);
                            }
                        }
                    }
                }
            });
        }
    }

    //点击纸牌调用
    private void flop(int num,final boolean needCoinAnimation) {
        if(boxState == BoxState.waitForChoose) {
            boxState = BoxState.showReward;
        }
        begin();
        boxes[num].boxFlop(reward,limitGoodsBallId,limitGoodsPenId,limitGoodsZombieId);
        reward(needCoinAnimation);
        update();
        openInt++;
    }
    public void flopAll(){
        for(int i = 0;i<4;i++){
            Group boxGroup = boxes[i].getGroup();
            if(boxGroup.findActor("back").isVisible()){
                flop(i,false);
            }
        }
    }
    int getRewardTime = 0;
    @Override
    public void begin() {
        super.begin();
        if(!showRewardInt.contains(reward,false)){
            if(getRewardTime>=1000) {
                reward = showRewardInt.get(0);
                showRewardInt.removeValue(reward,false);
                return;
            }
            getRewardTime++;
            begin();
        }else {
            showRewardInt.removeValue(reward,false);
        }
    }

    @Override
    protected void reward(boolean showAnimation) {
        super.reward(showAnimation);
    }


    @Override
    public void show(Group parent) {
        super.show(parent);
    }



    @Override
    public void update() {
//        super.update();
        group.findActor("btn_next").setVisible(false);
        group.findActor("btn_open_all").setVisible(false);
        for(Box box: boxes){
            if(openCost[openInt]>0 && !box.isOpen()) {
                box.getGroup().findActor("cost").setVisible(true);
                ((Label)box.getGroup().findActor("cost_font")).setText(openCost[openInt]+"");
            }else {
                box.getGroup().findActor("cost").setVisible(false);
            }
        }
        if(boxState == BoxState.waitForChoose){
            if(Configuration.videoAdReady) {
                group.findActor("btn_open_all").setVisible(true);
            }
        }
        if(boxState == BoxState.waitForChoose && openInt>=1){
            group.findActor("btn_next").setVisible(true);
        }
        if(boxState == BoxState.waitForChoose && openInt>=4){
            group.findActor("btn_open_all").setVisible(false);
        }
    }


    public enum BoxState{
        none,shuffle, waitForChoose,showReward,watchVideoToOpenAll,out
    }
}
