package com.zyb.loveball.dialogs;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.objects.Card;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.LClickListener;

/**
 * Created by Doodle on 2019/3/5.
 */

public class SuccessBonusCardDialog extends SuccessBonusBaseDialog{
    final int CARD_NUM = 4;
    Card[] cards = new Card[4];
    CardState cardState = CardState.none;
    @Override
    public void load(Group group) {
        super.load(group);
        init();
    }

    @Override
    public void init() {
        super.init();
        initLimitGoods(-1);
        initAnimation();
    }


    private void initAnimation() {
        for(int i = 0;i<CARD_NUM;i++){
            cards[i] = new Card((Group) group.findActor("card"+(i+1),Touchable.enabled),i+1);
            cards[i].setCardListener(new Card.CardListener() {
                @Override
                public boolean changeDialogState(CardState fromState, CardState targetState) {
                    return changeState(fromState,targetState);
                }

                @Override
                public void updateDialogCoin() {
                    updateCoin();
                }
            });
            float width = cards[i].getGroup().getWidth();
            float height = cards[i].getGroup().getHeight();
            cards[i].getGroup().setOrigin(width/2,height/2);
            cards[i].addShuffleAction(CARD_NUM);
        }
    }
    public boolean changeState(CardState fromState,CardState targetState){
        if(cardState == fromState){
            cardState = targetState;
            update();
            return true;
        }
        return false;
    }
    private int openInt = 0;
    private int[] openCost = {0,0,100,150,0};
    @Override
    protected void initButtons() {
        super.initButtons();
        initCardButton();
        group.findActor("btn_next",Touchable.enabled).addListener(new ButtonListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                GameScreen.canShowFullAds = false;

                    successBonusBaseDialogListener.next();
            }
        });
        group.findActor("btn_open_all",Touchable.enabled).addListener(new ButtonListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                successBonusBaseDialogListener.playVideo(BaseScreen.PendingAction.SECOND_BONUS,3);
//                flopAll();
            }
        });
    }

    private void initCardButton() {
        for(int i = 0;i<CARD_NUM;i++){
            final Group cardGroup = group.findActor("card"+(i+1), Touchable.enabled);
            final int cardInt = i;
            cardGroup.addListener(new LClickListener(){
                @Override
                public void touchDownEffect() {
                    super.touchDownEffect();
                    if(cardGroup.findActor("back").isVisible()){
                        if(cardState == CardState.waitForChoose){
                            if(Configuration.settingData.cost(openCost[openInt])) {
                                updateCoin();
                                flop(cardInt,false);
                            }
                        }
                    }
                }
            });
        }
    }

    //点击纸牌调用
    private void flop(int num,final boolean needCoinAnimation) {
        if(cardState == CardState.waitForChoose) {
            cardState = CardState.showReward;
        }
        begin();
        cards[num].cardFlop(reward,limitGoodsBallId,limitGoodsPenId,limitGoodsZombieId);
        reward(needCoinAnimation);
        update();
        openInt++;
    }
    public void flopAll(){
        for(int i = 0;i<4;i++){
            Group cardGroup = cards[i].getGroup();
            if(cardGroup.findActor("back").isVisible()){
                flop(i,false);
            }
        }
    }
    int getRewardTime = 0;
    @Override
    public void begin() {
        super.begin();
        if(!showRewardInt.contains(reward,false)){
            if(getRewardTime>=1000) {
                reward = showRewardInt.get(0);
                showRewardInt.removeValue(reward,false);
                return;
            }
            getRewardTime++;
            begin();
        }else {
            showRewardInt.removeValue(reward,false);
        }
    }

    @Override
    protected void reward(boolean showAnimation) {
        super.reward(showAnimation);
    }


    @Override
    public void show(Group parent) {
        super.show(parent);
    }



    @Override
    public void update() {
//        super.update();
        group.findActor("btn_next").setVisible(false);
        group.findActor("btn_open_all").setVisible(false);
        for(Card card:cards){
            if(openCost[openInt]>0 && !card.isOpen()) {
                card.getGroup().findActor("cost").setVisible(true);
                ((Label)card.getGroup().findActor("cost_font")).setText(openCost[openInt]+"");
            }else {
                card.getGroup().findActor("cost").setVisible(false);
            }
        }
        if(cardState == CardState.waitForChoose){
            if(Configuration.videoAdReady) {
                group.findActor("btn_open_all").setVisible(true);
            }
        }
        if(cardState == CardState.waitForChoose && openInt>=1){
            group.findActor("btn_next").setVisible(true);
        }
        if(cardState == CardState.waitForChoose && openInt>=4){
            group.findActor("btn_open_all").setVisible(false);
        }
    }


    public enum CardState{
        none,shuffle, waitForChoose,showReward,watchVideoToOpenAll,out
    }
}
