package com.zyb.loveball.dialogs;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.animations.CommonParticleEffect;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.listener.LClickListener;
import com.zyb.loveball.utils.listener.SoundButtonListener;

/**
 * Created by Doodle on 2019/1/29.
 */

public class SuccessBonusTurntableDialog extends SuccessBonusBaseDialog{
    private TurntableState turntableState = SuccessBonusTurntableDialog.TurntableState.firstReady;

    private Actor turntableDisk;
    private CommonAnimation titleAnimation;
    private CommonAnimation lampAnimation;
    private CommonAnimation lightAnimation;
    private CommonParticleEffect ribbonAnimation;
    private float endAngle;
    private float lampVel = 1f;

    @Override
    public void load(Group group) {
        super.load(group);
        turntableDisk = group.findActor("turntable_disk", Touchable.childrenOnly);
        titleAnimation = new CommonAnimation("animations/goodluck.skel");
        lampAnimation = new CommonAnimation("animations/deng.skel");
        lightAnimation = new CommonAnimation("animations/guang.skel");
        ribbonAnimation = new CommonParticleEffect("animations/particle/c3", Assets.instance.ui);
        init();
    }

    @Override
    public void init() {
        super.init();
        initAnimation();
        initLimitGoods(-1);
        begin();
    }

    @Override
    public void show(Group parent) {
        super.show(parent);
    }

    @Override
    protected void initLimitGoods(int type){
        super.initLimitGoods(type);
        for(int i = 0;i < 6;i++){
            Group itemGroup = group.findActor("turntable_disk_item"+(i+1));
            switch (showRewardInt.get(i)){
                case 1:
                    ((Label)itemGroup.findActor("item_font")).setText("+"+coinValue[1]);
                    break;
                case 2:
                    ((Label)itemGroup.findActor("item_font")).setText("+"+coinValue[2]);
                    break;
                case 3:
                    ((Label)itemGroup.findActor("item_font")).setText("+"+coinValue[3]);
                    break;
                case 4:
                    //85pifu
                    break;
                case 5:
                    //75pifu
                    break;
                case 6:
                    //xianshizhadan
                    initLimitGoodsUI(itemGroup,limitGoodsBallId);
                    break;
                case 7:
                    ((Label)itemGroup.findActor("item_font")).setText("+"+coinValue[7]);
                    break;
                case 8:
                    ((Label)itemGroup.findActor("item_font")).setText("+"+coinValue[8]);
                    break;
                case 9:
                    //xianshibi
                    initLimitGoodsUI(itemGroup,limitGoodsPenId);
                    break;
                case 10:
                    //xianshijiangshi
                    initLimitGoodsUI(itemGroup,limitGoodsZombieId);
                    break;
            }
        }
    }
    private void initLimitGoodsUI(Group itemGroup,int limitGoodsId){
        ((Label)itemGroup.findActor("item_font")).setText("FREE");
        Image image = itemGroup.findActor("item_skin_image");
        GoodData goodData;
        Sprite sprite;
        if(limitGoodsId <=100) {
            goodData = Assets.instance.getGoodData(limitGoodsId, 0);
            sprite = Assets.instance.ui.createSprite(goodData.getResources()+"_icon");
        }else if(limitGoodsId >100&& limitGoodsId <=1000){
            goodData = Assets.instance.getGoodData(limitGoodsId -100, 1);
            sprite = Assets.instance.ui.createSprite(goodData.getResources());
        }else{
            goodData = Assets.instance.getGoodData(limitGoodsId -1000, 2);
            sprite = Assets.instance.ui.createSprite(goodData.getResources()+"_icon");
        }
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        image.setDrawable(spriteDrawable);
        itemGroup.findActor("item_skin").setVisible(true);
        itemGroup.findActor("item_coin").setVisible(false);
    }
    private void initAnimation(){
        titleAnimation.setPosition(0,0);
        lampAnimation.setPosition(0,0);
        lightAnimation.setPosition(0,0);
        ribbonAnimation.setPosition(0, 800);
        titleAnimation.getState().setAnimation(0, "1", false);
        lampAnimation.getState().setAnimation(0, "1", true);
        lightAnimation.getState().setAnimation(0, "1", true);
        group.addActor(titleAnimation);
        ((Group)group.findActor("turntable")).addActorAfter(group.findActor("turntable_bg"),lampAnimation);
        ((Group)group.findActor("turntable")).addActorBefore(group.findActor("turntable_bg"),lightAnimation);
        group.addActorBefore(group.findActor("turntable"),ribbonAnimation);
        ribbonAnimation.setVisible(false);
        group.findActor("turntable").setScale(0.3f);
        group.findActor("turntable").addAction(Actions.scaleTo(1,1,0.6f,Interpolation.swingOut));
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        group.findActor("btn_turntable").addListener(new LClickListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if(turntableState == TurntableState.firstReady) {

                }else if(turntableState == TurntableState.secondReady){
                    //看视频旋转
                    if(Configuration.videoAdReady) {
                        successBonusBaseDialogListener.playVideo(BaseScreen.PendingAction.SECOND_BONUS, 0);
                    }
                }
            }
        });
        group.findActor("btn_next", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                GameScreen.canShowFullAds = false;
                    successBonusBaseDialogListener.next();
            }
        });
    }

    @Override
    public void begin() {
        super.begin();
        for(int i = 0;i<showRewardInt.size;i++){
            if(showRewardInt.get(i) == reward){
                endAngle = 360/6*i;
                Log.log("SuccessBonusTurntableDialog", "begin()----> endAngle " + endAngle);
                Log.log("SuccessBonusTurntableDialog", "begin()----> reward " + reward);
                Log.log("SuccessBonusTurntableDialog", "begin()----> i " + i);
            }
        }

        turntableDisk.addAction(Actions.sequence(Actions.run(turntableBeginRunnable),
                Actions.delay(0.5f),
                Actions.run(lampAddRunnable),
                Actions.rotateBy(-360,0.75f,Interpolation.exp5In),
                Actions.run(lampAddRunnable),
                Actions.rotateBy(-360*6,0.75f),
                Actions.run(lampSubRunnable),
                Actions.rotateBy(-360*2-endAngle,2.5f,Interpolation.exp10Out),
                Actions.run(turntableEndRunnable)
                ));
    }

    private Runnable lampAddRunnable = new Runnable() {
        @Override
        public void run() {
            lampVel+=0.5f;
            lampAnimation.getState().setTimeScale(lampVel);
            Log.log("successBonusTurntableDialog","lampAnimation vel scale------->"+lampVel);
        }
    };
    private Runnable lampSubRunnable = new Runnable() {
        @Override
        public void run() {
            lampVel--;
            lampAnimation.getState().setTimeScale(lampVel);
            Log.log("successBonusTurntableDialog","lampAnimation vel scale------->"+lampVel);
        }
    };

    private Runnable turntableBeginRunnable = new Runnable() {
        @Override
        public void run() {
            turntableDisk.setRotation(0);
            if(turntableState == TurntableState.firstReady) {
                turntableState = TurntableState.firstStart;
                update();
            }else if(turntableState == TurntableState.secondReady){
                turntableState = TurntableState.secondStart;
                update();
            }
        }
    };
    private Runnable turntableEndRunnable = new Runnable() {
        @Override
        public void run() {
            if(turntableState == TurntableState.firstStart){
                turntableState = TurntableState.secondReady;
            }else if(turntableState == TurntableState.secondStart){
                turntableState = TurntableState.secondEnd;
            }
            reward(true);
            update();
        }
    };



    @Override
    public void update() {
//        super.update();
        switch (turntableState){
            case firstReady:
                group.findActor("first_font").setVisible(true);
                group.findActor("second_font").setVisible(false);
                ribbonAnimation.setVisible(false);
                if(lampAnimation!=null){
                    lampAnimation.getState().setAnimation(0, "1", true);
                }
                break;
            case firstStart:
                group.findActor("first_font").setVisible(true);
                group.findActor("second_font").setVisible(false);
                ribbonAnimation.setVisible(false);
                if(lampAnimation!=null){
                    lampAnimation.getState().setAnimation(0, "2", true);
                }
                break;
            case secondReady:
                group.findActor("btn_next").setVisible(true);
                group.findActor("first_font").setVisible(true);
                if(Configuration.videoAdReady) {
                    group.findActor("second_font").setVisible(true);
                    group.findActor("first_font").setVisible(false);
                }
                ribbonAnimation.setVisible(true);
                if(lampAnimation!=null){
                    lampAnimation.getState().setAnimation(0, "3", true);
                }
                break;
            case secondStart:
                group.findActor("first_font").setVisible(true);
                group.findActor("second_font").setVisible(false);
                ribbonAnimation.setVisible(false);
                if(lampAnimation!=null){
                    lampAnimation.getState().setAnimation(0, "2", true);
                }
                break;
            case secondEnd:
                group.findActor("first_font").setVisible(true);
                group.findActor("second_font").setVisible(false);
                ribbonAnimation.setVisible(true);
                if(lampAnimation!=null){
                    lampAnimation.getState().setAnimation(0, "3", true);
                }
                break;
        }
    }

    public enum TurntableState{
        firstReady,firstStart,secondReady,secondStart,secondEnd
    }
}
