package com.zyb.loveball.dialogs;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.AnimationState;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.animations.CommonParticleEffect;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.audio.AudioProcess;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.listeners.FullScreenListener;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.ChallengeScreen;
import com.zyb.loveball.screens.LevelScreen;
import com.zyb.loveball.screens.MenuScreen;
import com.zyb.loveball.screens.StoreScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.LayerUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.LClickListener;
import com.zyb.loveball.utils.listener.SoundButtonListener;

/**
 * Created by Doodle on 2018/9/13.
 */

public class SuccessDialog extends BaseDialog {
    SuccessDialogListener successDialogListener;
    Label coinRewardLabel;
    Group videoCoinReward;
    Group videoSkinReward;
    Group videoLimitReward;
    Group videoTurntableReward;
    Group videoCardReward;
    Group videoBoxReward;
    CommonAnimation starAnimation;
    SuccessDialogState successDialogState = SuccessDialogState.playing_animation;

    @Override
    public void show(Group parent) {
        super.show(parent);
        group.getActions().clear();
//        group.setTouchable(Touchable.disabled);
        initSettingData();
        group.setPosition(0, (HelloZombieGame.getViewport().getWorldHeight() - 800) / 2);
        Configuration.settingData.unlockNextLevel(GameInfo.chapter, GameInfo.id,GameInfo.type);
        init();
        group.addAction(Actions.sequence(Actions.scaleTo(1, 1, 0.1F), Actions.run(new Runnable() {
            @Override
            public void run() {
                start();
            }
        })));
    }

    private void init(){
        coinLabel = group.findActor("coin_font");
        coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
//        AudioProcess.playSound("win" + GameInfo.star, 1);
        videoCoinReward = group.findActor("video_coin_reward");
        videoSkinReward = group.findActor("video_skin_reward");
        videoLimitReward = group.findActor("video_skin_limited_reward");
        videoTurntableReward = group.findActor("video_turntable_reward");
        videoCardReward = group.findActor("video_card_reward");
        videoBoxReward = group.findActor("video_box_reward");
        coinRewardLabel = group.findActor("coin_value");
        group.findActor("coin_reward").setVisible(false);
        videoSkinReward.setVisible(false);
        videoCoinReward.setVisible(false);
        videoLimitReward.setVisible(false);
        videoTurntableReward.setVisible(false);
        videoCardReward.setVisible(false);
        videoBoxReward.setVisible(false);
        buttonGroupVisible(false, 0);
    }

    private void start(){
        checkAds();
        screenShot();
    }

    private void checkAds(){
        boolean showAd;
        if(GameInfo.chapter == 2 && GameInfo.id == 10 && GameInfo.oldStar == 0){
            successDialogListener.showChallengeConfirmDialog();
            showAd = false;
        }else {
            showAd = BaseScreen.showFullScreenAd(0.1f, new FullScreenListener() {
                @Override
                public void adClosed() {
                    initAnimation();
                    checkReward();
                    AudioProcess.playSound("win" + GameInfo.star, 1);
                }
            });
        }
        if(!showAd) {
            initAnimation();
            checkReward();
            AudioProcess.playSound("win" + GameInfo.star, 1);

        }
    }


    private void screenShot() {
        String pngFileName = "level_pic_" + GameInfo.levelFileName;
        FileHandle file = Gdx.files.local(Constant.LEVEL_PIC_PATH + pngFileName + ".png");
        MScreenUtil.writePngFromBuffer(file, Constant.SCREEN_SHOT_W_BIG, Constant.SCREEN_SHOT_H_BIG, true);
        HelloZombieGame.resumeInputProcessor();
    }

    private void buttonGroupVisible(boolean groupVisible, int delaySecond) {
        Group buttons = group.findActor("buttons");
        if (delaySecond == 0) {
            buttons.setVisible(groupVisible);
        } else if (delaySecond > 0) {
            buttons.addAction(Actions.sequence(Actions.delay(delaySecond), Actions.visible(groupVisible)));
        }
        if (GameInfo.chapter == 1) {
            buttons.findActor("btn_home").setVisible(false);
            buttons.findActor("btn_level").setVisible(false);
            buttons.findActor("btn_skin").setVisible(false);
        }
        if(GameInfo.star == 3){
            buttons.findActor("btn_retry").setVisible(false);
            buttons.findActor("btn_next").setX(buttons.getWidth()/2,Align.center);
        }
    }

    private void initSettingData() {
        if(GameInfo.type == 0) {
            int id = GameInfo.id + 1;
            int chapter = GameInfo.chapter;
            if (id > 10) {
                id = 1;
                chapter++;
                if (chapter == Constant.CHAPTER_NUN + 1) {
                    chapter = 1;
                }
            }
            Configuration.settingData.setLastLevelInt(chapter, id, GameInfo.type);
        }else if(GameInfo.type == 1){
            int id = GameInfo.id + 1;
            int chapter = GameInfo.chapter;
            if (id > 10) {
                id = 1;
                chapter++;
                int chapterSub = chapter-1000;
                if (chapterSub == Constant.HAPPY_GLASS_CHAPTER_NUN + 1) {
                    chapter = 1001;
                }
            }
            Configuration.settingData.setLastLevelInt(chapter, id, GameInfo.type);
        }
    }

    public void setSuccessDialogListener(SuccessDialogListener successDialogListener) {
        this.successDialogListener = successDialogListener;
     //   successDialogListener.showFullScreenAd(0.1f);
    }

    private void checkReward() {
        if (GameInfo.oldStar == 0) {
            if (Configuration.settingData.isFirstPlay()) {
                FlurryUtils.level_FirstWin(GameInfo.chapter, GameInfo.id);
            }
            baseReward();
            buttonGroupVisible(true, 0);
//            noReward();
//            extraReward();
        } else {
            noReward();
        }
    }

    private void extraReward() {
//        boolean coinEnough = false;
//        boolean hasLimitBall = false;
        FlurryUtils.Event_CoinPush("All");
        final boolean course;
        if(GameInfo.chapter == 1 && GameInfo.id <= 1){
            course = true;
        }else {
            course = false;
        }
        if (!Configuration.videoAdReady && !course) {
            Log.log("SuccessDialog", "extraReward()----> video reward ad is not ready");
            buttonGroupVisible(true, 0);
            return;
        }
//        if(Configuration.checkDiscountBallId(0.85F) !=-1){
//            coinEnough = true;
//        }
//        if(Configuration.checkLimitId() != -1){
//            hasLimitBall = true;
//        }
        int rewardId = GameInfo.levelData.getPushId();
        Log.log("SuccessDialog", "extraReward()----> rewardId " + rewardId);
        int delaySeconds = 0;
        if(GameInfo.levelData.getPushType() == 1) {
            delaySeconds = extraRewardOne(rewardId);
        } else if(GameInfo.levelData.getPushType() == 2){
            delaySeconds = extraRewardTwo(course);
        } else if(GameInfo.levelData.getPushType() == 3){
            delaySeconds = extraRewardThree(course);
        } else if(GameInfo.levelData.getPushType() == 4){
            delaySeconds = extraRewardFour(course);
        }
        buttonGroupVisible(true, delaySeconds);
    }
    private int extraRewardOne(int rewardId) {
        int delaySeconds;
        if (rewardId == 0) {
            delaySeconds = 0;
        } else if (rewardId <= 3 || rewardId == 7 || rewardId == 8) {
            initCoinReward(rewardId);
            delaySeconds = 1;
        } else if (rewardId <= 5) {
            initSkinReward(rewardId);
            delaySeconds = 1;
        } else {
            initSkinLimitReward(rewardId);
            delaySeconds = 1;
        }
        return delaySeconds;
    }

    private int extraRewardTwo(final boolean course){
        if(course) {
            initCourse(videoTurntableReward);
        }
        videoTurntableReward.setScale(4);
        videoTurntableReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
        videoTurntableReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                if(course){
                    successDialogListener.showSuccessBonusTurntableDialog();
                    videoTurntableReward.setVisible(false);
                    group.findActor("layer").setVisible(false);
                    group.findActor("handAnimation").setVisible(false);
                }else {
                    pendingReward = 2;
                    successDialogListener.playVideo(BaseScreen.PendingAction.SUCCESS_BONUS, pendingReward);
                    videoTurntableReward.setVisible(false);
                }
            }
        });
        return 1;
    }
    private int extraRewardThree(final boolean course){
        if(course) {
            initCourse(videoCardReward);
        }
        videoCardReward.setScale(4);
        videoCardReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
        videoCardReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                if(course){
                    successDialogListener.showSuccessBonusCardDialog();
                    videoCardReward.setVisible(false);
                    group.findActor("layer").setVisible(false);
                    group.findActor("handAnimation").setVisible(false);
                }else {
                    pendingReward = 3;
                    successDialogListener.playVideo(BaseScreen.PendingAction.SUCCESS_BONUS, pendingReward);
                    videoCardReward.setVisible(false);
                }
            }
        });
        return 1;
    }
    private int extraRewardFour(final boolean course){
        if(course) {
            initCourse(videoBoxReward);
        }
        videoBoxReward.setScale(4);
        videoBoxReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
        videoBoxReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                if(course){
                    successDialogListener.showSuccessBonusBoxDialog();
                    videoBoxReward.setVisible(false);
                    group.findActor("layer").setVisible(false);
                    group.findActor("handAnimation").setVisible(false);
                }else {
                    pendingReward = 4;
                    successDialogListener.playVideo(BaseScreen.PendingAction.SUCCESS_BONUS, pendingReward);
                    videoBoxReward.setVisible(false);
                }
            }
        });
        return 1;
    }
    private void initCourse(Group initGroup){
        Image layer = LayerUtils.createLayer(0.6f);
        layer.setName("layer");
        group.addActorBefore(group.findActor("video_coin_reward"),layer);
        starAnimation.getState().setTimeScale(20);
        AudioProcess.stopAllSound();

        adImageInvisible(initGroup);

        final CommonAnimation handAnimation = new CommonAnimation("animations/dianji.skel", 0.5f);
        handAnimation.getState().setAnimation(0,"2",true);
        handAnimation.setName("handAnimation");
        group.addActor(handAnimation);
        Actor videoAct = group.findActor("video_coin_reward");
        float x = videoAct.getX();
        float y = videoAct.getY();
        handAnimation.setPosition(x, y);
        handAnimation.setxOffset(videoAct.getWidth() / 2);
        handAnimation.setyOffset(videoAct.getHeight() / 2);
        handAnimation.setVisible(false);
        handAnimation.addAction(Actions.sequence(Actions.delay(1f),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        handAnimation.setVisible(true);
                    }
                })));
    }

    private void adImageInvisible(Group initGroup) {
        initGroup.findActor("Image_8").setVisible(false);
        for(Actor actor:initGroup.getChildren()){
            if(actor.getName().equals("Image_5")) continue;
            actor.setX(actor.getX()+35);
            if(actor.getName().startsWith("success_")){
                actor.setX(actor.getX()-15);
            }
        }
    }


    private void baseReward() {
        group.findActor("coin_reward").setVisible(true);
        Configuration.settingData.addCoin(GameInfo.levelData.getReward());
        coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        coinRewardLabel.setText(Configuration.numType.format(GameInfo.levelData.getReward()));
    }

    private void noReward() {
        group.findActor("coin_reward").setVisible(false);
        buttonGroupVisible(true, 0);
    }

//    @Override
//    public void update() {
//        super.update();
//    }

    private int pendingReward;

    private void initCoinReward(final int rewardId) {
        int[] rewardNum = Constant.rewardNum;
        if (rewardId == 1) {
//            pendingReward = 10 * GameInfo.levelData.getReward();
            pendingReward = rewardNum[1];
            FlurryUtils.Event_CoinPush("Coin10");
        } else if (rewardId == 2) {
//            pendingReward = 8 * GameInfo.levelData.getReward();
            pendingReward = rewardNum[2];
            FlurryUtils.Event_CoinPush("Coin8");
        } else if (rewardId == 3) {
//            pendingReward = 5 * GameInfo.levelData.getReward();
            pendingReward = rewardNum[3];
            FlurryUtils.Event_CoinPush("Coin5");
        }
        else if (rewardId == 7) {
//            pendingReward = 5 * GameInfo.levelData.getReward();
            pendingReward = rewardNum[7];
            FlurryUtils.Event_CoinPush("Coin2");
        }
        else if (rewardId == 8) {
//            pendingReward = 5 * GameInfo.levelData.getReward();
            pendingReward = rewardNum[8];
            FlurryUtils.Event_CoinPush("Coin4");
        }
        Label label = group.findActor("coin_times");
        label.setText("+" + Configuration.numType.format(pendingReward));
        videoCoinReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                successDialogListener.playVideo(BaseScreen.PendingAction.SUCCESS_COIN, pendingReward);
                FlurryUtils.Event_CoinClick("Coin" + (pendingReward));
                videoCoinReward.setVisible(false);
            }
        });

        videoCoinReward.setScale(4);
        videoCoinReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
    }

    private void initSkinLimitReward(final int rewardId) {
        final int id;
        if(rewardId == 6){
            id = Configuration.checkLimitBallId();
        }else if(rewardId == 9){
            id = Configuration.checkLimitPenId();
        }else if(rewardId == 10){
            id = Configuration.checkLimitZombieId();
        }else {
            id = -1;
        }
        if (id == -1) return;
        Image image = videoLimitReward.findActor("skin_mask");
        GoodData goodData;
        Sprite sprite;
        if(id<=100) {
            goodData = Assets.instance.getGoodData(id, 0);
            sprite = Assets.instance.ui.createSprite(goodData.getResources()+"_icon");
        }else if(id>100&&id<=1000){
            goodData = Assets.instance.getGoodData(id-100, 1);
            sprite = Assets.instance.ui.createSprite(goodData.getResources());
        }else{
            goodData = Assets.instance.getGoodData(id-1000, 2);
            sprite = Assets.instance.ui.createSprite(goodData.getResources()+"_icon");
        }
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        image.setDrawable(spriteDrawable);
        videoLimitReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                FlurryUtils.Event_CoinClick("SkinFree");
                successDialogListener.playVideo(BaseScreen.PendingAction.LIMIT_GOODS, id);
                videoLimitReward.setVisible(false);
            }
        });
        videoLimitReward.setScale(4);
        FlurryUtils.Event_CoinPush("SkinFree");
        videoLimitReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
    }

    private void initSkinReward(final int rewardId) {
        float discount = 1;
        if (rewardId == 4) {
            discount = 0.85F;
        } else if (rewardId == 5) {
            discount = 0.75F;
            TextureRegion region = Assets.instance.ui.findRegion("discount_25");
            Image image = group.findActor("discount_pic");
            TextureRegionDrawable drawable = (TextureRegionDrawable) image.getDrawable();
            drawable.setRegion(region);
            image.setSize(region.getRegionWidth() / 2, region.getRegionHeight() / 2);
        }
        final int id = Configuration.checkDiscountBallId(discount);
        if (id == -1) return;
        final GoodData goodData = Assets.instance.getGoodData(id, 1);
        final int discountValue = (int) (discount * goodData.getCoin());
        ((Label) group.findActor("cost_money")).setText("" + discountValue);
        ((Label) group.findActor("cost_discount")).setText("" + goodData.getCoin());
        Image image = videoSkinReward.findActor("skin_mask");
        Sprite sprite = Assets.instance.ui.createSprite(goodData.getResources());
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        image.setDrawable(spriteDrawable);
        videoSkinReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                FlurryUtils.Event_CoinClick("SkinDiscount");
                successDialogListener.playVideo(BaseScreen.PendingAction.BALL_UNLOCK, id * Constant.BALLID_PREFIX + discountValue);
                videoSkinReward.setVisible(false);
            }
        });
        videoSkinReward.setScale(4);
        FlurryUtils.Event_CoinPush("SkinDiscount");
        videoSkinReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
    }

    public void initAnimation() {
        starAnimation = new CommonAnimation("animations/jiesuan.skel", 0.5f);
        starAnimation.getState().addListener(new AnimationState.AnimationStateAdapter() {
            @Override
            public void complete(AnimationState.TrackEntry entry) {
                super.complete(entry);
                Log.log("SuccessDialog","initAnimation--------->complete()");
                successDialogState = SuccessDialogState.ready;
//                group.setTouchable(Touchable.enabled);
            }
        });
        group.addActorAfter(layer, starAnimation);
        starAnimation.setPosition(0, 0);
        starAnimation.getState().setAnimation(0, GameInfo.star + "", false);
        if (!Configuration.poor) {
            CommonParticleEffect effect = new CommonParticleEffect("animations/particle/c3", Assets.instance.ui);
            effect.setPosition(0, 800);
            group.addActorAfter(layer, effect);
        }
    }

    @Override
    protected void initButtons() {
        group.findActor("coin_group", Touchable.enabled).addListener(new LClickListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                successDialogListener.showCoinDialog();
            }
        });
        group.findActor("btn_next", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                GameScreen.canShowFullAds = false;
                successDialogListener.next();
            }
        });
        group.findActor("btn_home", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(MenuScreen.class);
            }
        });
        group.findActor("btn_level", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if (GameInfo.challenge) {
                    baseListener.screenEnd(ChallengeScreen.class);
                } else {
                    baseListener.screenEnd(LevelScreen.class);
                }
            }
        });
        group.findActor("btn_skin", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(StoreScreen.class);
            }
        });
        group.findActor("btn_retry", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(GameScreen.class);
//                successDialogListener.showFullScreenAd(0.2f);
            }
        });
    }

    int srcReward;
    int dstReward;

    public void extraCoinGot() {
        srcReward = GameInfo.levelData.getReward();
        dstReward = srcReward + pendingReward;
        Configuration.settingData.addCoin(pendingReward);
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                if (srcReward < dstReward - 7) {
                    srcReward += (int)(Math.random()*3)+4;
                } else {
                    srcReward = dstReward;
                    extraCoinEnd();
                    coinRewardLabel.getActions().clear();
                }
                coinRewardLabel.setText(srcReward + "");
            }
        };
        if (coinRewardLabel != null) {
            coinRewardLabel.addAction(Actions.forever(Actions.sequence(Actions.run(runnable), Actions.delay(0.01F))));
        }
    }

    private void extraCoinEnd() {
        final CommonAnimation coin = new CommonAnimation("animations/jinbi_jiesuan.skel", 0.5F, "jinbi_fly");
        coin.getState().setTimeScale(1.5F);
        coin.getState().setAnimation(0, "2", false);
        coin.getState().addListener(new AnimationState.AnimationStateAdapter() {
            @Override
            public void complete(AnimationState.TrackEntry entry) {
                super.complete(entry);
                coin.remove();
                coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
            }
        });
        group.addActor(coin);
    }
    public enum SuccessDialogState{
        playing_animation,ready
    }

    @Override
    public void update() {
    }

    public interface SuccessDialogListener {
    public void next();

    public void showCoinDialog();


    public void playVideo(BaseScreen.PendingAction action, int pendingValue);

    public void showSuccessBonusTurntableDialog();

    public void showSuccessBonusCardDialog();

    public void showSuccessBonusBoxDialog();

    public void showChallengeConfirmDialog();
    }
}
