package com.zyb.loveball.objects;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyContactAdapter;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.EdgeShape;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.utils.Array;
import com.zyb.loveball.GamePanel;

public class Border extends BaseObject {

    Vector2 position;

    public Border(Body body, GamePanel panel) {
        super(panel);
        this.body = body;
        Array<Fixture> fixtures = new Array<>();
        fixtures.addAll(body.getFixtureList());
        initBounds(fixtures);
    }

    private void initBounds(Array<Fixture> fixtures) {
        float x1 = 3-gamePanel.getCamera().zoom*3;
        float y1 = 5-gamePanel.getCamera().zoom*5-(gamePanel.getCamera().zoom-1)*(gamePanel.getCamera().viewportHeight-10)/2;
        float x2 = x1+gamePanel.getCamera().viewportWidth*gamePanel.getCamera().zoom;
        float y2 = y1+gamePanel.getCamera().viewportHeight*gamePanel.getCamera().zoom;
//        System.out.println("viewportWidth:"+gamePanel.getCamera().viewportWidth);
//        System.out.println("viewportHeight:"+gamePanel.getCamera().viewportHeight);
//        System.out.println("x1:"+x1);
//        System.out.println("y1:"+y1);
//        System.out.println("x2:"+x2);
//        System.out.println("y2:"+y2);
        body.setTransform(0, 0, 0);
        for (Fixture temp : fixtures) {
            body.destroyFixture(temp);
        }
        FixtureDef fixtureDef = new FixtureDef();
        fixtureDef.isSensor = true;
        EdgeShape shape = new EdgeShape();
        fixtureDef.shape = shape;
        float offset = 0.2F;
        shape.set(x1 - offset, y1 - offset, x1 - offset, y2 + offset);
        body.createFixture(fixtureDef);
        shape.set(x1 - offset, y1 - offset, x2 + offset, y1 - offset);
        body.createFixture(fixtureDef);
        shape.set(x2 + offset, y1- offset, x2 + offset, y2 + offset);
        body.createFixture(fixtureDef);
        shape.set(x1 - offset, y2 + offset, x2 + offset, y2 + offset);
        body.createFixture(fixtureDef);
        body.setContactListener(new BodyContactAdapter() {
            public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
                super.beginContact(contact, fixtureA, fixtureB, isA);
                Body tmpBody;
                if (isA) {
                    tmpBody = fixtureB.getBody();
                } else {
                    tmpBody = fixtureA.getBody();
                }
                String name = gamePanel.getScene().getName(tmpBody);
                if (name != null) {
                    if (name.startsWith("ball")) {

//                        gamePanel.getBall().bomb(false,null);
                        ((Ball)tmpBody.getUserData()).bomb(false,null);

                    } else if (name.startsWith("heart")) {
//                        gamePanel.getZombie().boom(1);
                        ((Zombie)tmpBody.getUserData()).boom(false,1);
                    }
                }
            }
        });
    }

}
