package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.SuccessBonusBoxDialog;

/**
 * Created by Doodle on 2019/3/27.
 */

public class Box {
    boolean open = false;
    Group group;
    BoxListener boxListener;
    int boxId = 1;

    public Box(Group group, int boxId) {
        this.group = group;
        this.boxId = boxId;
    }

    public Group getGroup() {
        return group;
    }

    public void setBoxListener(BoxListener boxListener) {
        this.boxListener = boxListener;
    }

    public void addShuffleAction(int boxNum) {
        group.addAction(Actions.sequence(
                Actions.run(noneToShuffle),
                Actions.moveTo(480, 0, 0),
                Actions.delay(boxId / 10f),
                Actions.parallel(Actions.moveTo(group.getX(), group.getY(), 0.3f),
                        Actions.rotateBy(360, 0.3f)),
                Actions.delay((boxNum - boxId) / 10f),
                Actions.moveTo(240 - group.getWidth() / 2, 400 - group.getHeight() / 2, 0.1f),
                Actions.moveBy((boxId - (boxNum + 1f) / 2) * 60, 0, 0.1f),
                Actions.moveTo(240 - group.getWidth() / 2, 400 - group.getHeight() / 2, 0.1f),
                Actions.moveBy(((boxNum + 1f - boxId) - (boxNum + 1f) / 2) * 60, 0, 0.1f),
                Actions.moveTo(240 - group.getWidth() / 2, 400 - group.getHeight() / 2, 0.1f),
                Actions.moveBy((boxId - (boxNum + 1f) / 2) * 60, 0, 0.1f),
                Actions.moveTo(240 - group.getWidth() / 2, 400 - group.getHeight() / 2, 0.1f),
                Actions.moveBy(((boxNum + 1f - boxId) - (boxNum + 1f) / 2) * 60, 0, 0.1f),
                Actions.moveTo(240 - group.getWidth() / 2, 400 - group.getHeight() / 2, 0.1f),
                Actions.moveTo(group.getX(), group.getY(), 0.2f),
                Actions.run(shuffleToWaitForFirstChooseRunnable))
        );
    }

    public void boxFlop(final int reward, int ball, int pen, int zombie) {
        open = true;
        initRewardUI(reward, ball, pen, zombie);
        group.findActor("back").setVisible(false);
        group.findActor("front").setVisible(true);
        group.findActor("item").addAction(Actions.sequence(
                Actions.scaleTo(0.1f,0.1f,0),
                Actions.parallel(
                        Actions.scaleTo(1f,1f,0.3f,Interpolation.swingOut),
                        Actions.moveBy(0,70,0.3f,Interpolation.circleOut)

                        ),
                Actions.moveBy(0,-70,0.5f, Interpolation.bounceOut),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        coinFly();
                        boxListener.changeDialogState(SuccessBonusBoxDialog.BoxState.showReward, SuccessBonusBoxDialog.BoxState.waitForChoose);
                    }

                    protected void coinFly() {
                        if (reward == 1 || reward == 2 || reward == 3 || reward == 7 || reward == 8) {
                            for (int i = 0; i < 5; i++) {
                                Image image = new Image(Assets.instance.ui.findRegion("coin"));
                                image.setScale(0.5f);
                                group.addActor(image);
                                image.setPosition(group.getWidth() / 2 - image.getWidth() / 4, group.getHeight() / 2 - image.getHeight() / 4);
                                image.addAction(Actions.sequence(
                                        Actions.delay(0.1f * i),
                                        Actions.moveTo(10 - group.getX(), 758 - group.getY(), 0.5f),
                                        Actions.run(new Runnable() {
                                            @Override
                                            public void run() {
                                                boxListener.updateDialogCoin();
                                            }
                                        })
                                ));
                            }
                        }
                    }
                })
        ));
    }

    private void initRewardUI(int reward, int ball, int pen, int zombie) {
        switch (reward) {
            case 1:
                ((Label) group.findActor("item_font")).setText("+" + Constant.rewardNum[1]);
                break;
            case 2:
                ((Label) group.findActor("item_font")).setText("+" + Constant.rewardNum[2]);
                break;
            case 3:
                ((Label) group.findActor("item_font")).setText("+" + Constant.rewardNum[3]);
                break;
            case 4:
                //85pifu
                break;
            case 5:
                //75pifu
                break;
            case 6:
                //xianshizhadan
                initLimitGoodsUI(ball);
                break;
            case 7:
                ((Label) group.findActor("item_font")).setText("+" + Constant.rewardNum[7]);
                break;
            case 8:
                ((Label) group.findActor("item_font")).setText("+" + Constant.rewardNum[8]);
                break;
            case 9:
                //xianshibi
                initLimitGoodsUI(pen);
                break;
            case 10:
                //xianshijiangshi
                initLimitGoodsUI(zombie);
                break;
        }
    }

    private void initLimitGoodsUI(int limitGoodsId) {
        Image image = group.findActor("item_skin_image");
        GoodData goodData;
        Sprite sprite;
        if (limitGoodsId <= 100) {
            goodData = Assets.instance.getGoodData(limitGoodsId, 0);
            sprite = Assets.instance.ui.createSprite(goodData.getResources() + "_icon");
            ((Label) group.findActor("item_font")).setText("Bomb\nSkin");
        } else if (limitGoodsId > 100 && limitGoodsId <= 1000) {
            goodData = Assets.instance.getGoodData(limitGoodsId - 100, 1);
            sprite = Assets.instance.ui.createSprite(goodData.getResources());
            ((Label) group.findActor("item_font")).setText("Pen\nSkin");
        } else {
            goodData = Assets.instance.getGoodData(limitGoodsId - 1000, 2);
            sprite = Assets.instance.ui.createSprite(goodData.getResources() + "_icon");
            ((Label) group.findActor("item_font")).setText("Zombie\nSkin");
        }
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        image.setDrawable(spriteDrawable);
        group.findActor("item_skin").setVisible(true);
        group.findActor("item_coin").setVisible(false);
    }

    public boolean isOpen() {
        return open;
    }


    Runnable shuffleToWaitForFirstChooseRunnable = new Runnable() {
        @Override
        public void run() {
            boxListener.changeDialogState(SuccessBonusBoxDialog.BoxState.shuffle, SuccessBonusBoxDialog.BoxState.waitForChoose);
        }
    };
    Runnable noneToShuffle = new Runnable() {
        @Override
        public void run() {
            boxListener.changeDialogState(SuccessBonusBoxDialog.BoxState.none, SuccessBonusBoxDialog.BoxState.shuffle);
        }
    };

    public interface BoxListener {
        public boolean changeDialogState(SuccessBonusBoxDialog.BoxState fromState, SuccessBonusBoxDialog.BoxState targetState);

        public void updateDialogCoin();
    }
}
