package com.zyb.loveball.objects;

import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.RepeatAction;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;

/**
 * Created by Doodle on 2019/2/19.
 */

public class NoviceCourse extends Group {
    public CourseState courseState;
    Group courseGroup;
    final int COURSE_NUM = 4;
    final String[] COURSE_NAME = {"ink_progress","hint","retry","net"};
    Label[] labels = new Label[COURSE_NUM];
    String[] labelString = {"The progress bar indicates the amount of ink in the brush. The less ink you use, the easier it is to get three stars.",
            "If you fail to complete a level or you can't get 3 stars,tap the hint button!",
            "You can click the Retry button to redo it.",
            "You can't draw lines in the red net."};

    public NoviceCourse(GameScreen screen, Group root) {
        courseGroup = screen.parseScene("ui/objects/ink_progress_course.json");
        courseGroup.setName("course");
        courseState = CourseState.none;
        update();
        addActor(courseGroup);
        for(int i = 0;i<labels.length;i++){
            labels[i] = courseGroup.findActor(COURSE_NAME[i]+"_text");
        }
        RepeatAction[] repeatAction = new RepeatAction[COURSE_NUM];
        for(int i = 0;i<repeatAction.length;i++){
            repeatAction[i] = Actions.forever(Actions.sequence(Actions.moveBy(-5,5,0.5f),
                    Actions.moveBy(5,-5,0.5f)));
        }
        for(int i = 0;i<COURSE_NUM;i++){
            courseGroup.findActor(COURSE_NAME[i]+"_arrow").addAction(repeatAction[i]);
        }
        for (int i = 0; i < labels.length; i++) {
            labels[i].setText(labelString[i]);
        }
        for (int i = 0; i < labels.length; i++) {
            labels[i].setAlignment(Align.center);
            float ff = labels[i].getX(Align.center);
            labels[i].setWidth(290);
            labels[i].setX(ff, Align.center);
            labels[i].setWrap(true);
        }
    }
    public void update(){
        for(Actor actor : courseGroup.getChildren()){
            actor.setVisible(false);
        }
        switch (courseState){
            case none:
                break;
            case ink_progress_course:
                courseGroup.findActor("ink_progress_course").setVisible(true);
                break;
            case hint_course:
                courseGroup.findActor("hint_course").setVisible(true);
                break;
            case retry_course:
                courseGroup.findActor("retry_course").setVisible(true);
                break;
            case net_course:
                courseGroup.findActor("net_course").setVisible(true);
                courseGroup.findActor("net_course").setY(-440-(Configuration.height-800));

        }
        for(Actor actor : courseGroup.getChildren()){
            if(actor.isVisible()){
                actor.setScale(0.3f);
                actor.addAction(Actions.scaleTo(1,1,0.3f, Interpolation.swingOut));
            }
        }
    }
    public void setNone(){
        courseState = CourseState.none;
        update();
    }

    public void noviceCourseCheck(float unActiveTime, GamePanel.GameState state) {

            if (GameInfo.chapter > 1 || GameInfo.id >= 3) {

                if (!Configuration.settingData.isInkProgressCourse()) {
                    Configuration.settingData.setInkProgressCourse(true);
                    courseState = CourseState.ink_progress_course;
                    update();
                }
            }
            if (GameInfo.chapter > 1 || GameInfo.id >= 3) {
                if (!Configuration.settingData.isHintCourse()) {
                    if (unActiveTime > Constant.hintCourseSecond && !GameInfo.hint && state == GamePanel.GameState.ready) {
                        Configuration.settingData.setHintCourse(true);
                        courseState = CourseState.hint_course;
                        update();
                    }
                }
                if (!Configuration.settingData.isRetryCourse()) {
                    if (unActiveTime > Constant.retryCourseSecond && state == GamePanel.GameState.gaming) {
                        Configuration.settingData.setRetryCourse(true);
                        courseState = CourseState.retry_course;
                        update();
                    }
                }
            }
//            if(GameInfo.chapter == 4 && GameInfo.id == 1){
//                if (!Configuration.settingData.isNetCourse()) {
//                    Configuration.settingData.setNetCourse(true);
//                    courseState = CourseState.net_course;
//                    update();
//                }
//            }
    }
    public enum CourseState {
        none, ink_progress_course, hint_course, retry_course, net_course
    }
}
