package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyContactAdapter;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.joints.RevoluteJointDef;
import com.badlogic.gdx.utils.Array;
import com.rube.WorldData;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.animations.FlyZombieAnimation;
import com.zyb.loveball.animations.StandZombieAnimation;
import com.zyb.loveball.animations.ZombieAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.attributes.ZombieAttribute;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.Log;

public class Zombie extends BaseObject {
    Body jointBody;
    TextureRegion region;
    String bodyName;
    Vector2 position;
    Vector2 bodyLinearVelocity;
    ZombieAnimation zombieAnimation;
    CommonAnimation zombieSomkeAnimation;
    private ZombieListener zombieListener;
    Vector2 tmpVector2 = new Vector2();
    ZombieState zombieState = ZombieState.none;
    ZombieBoom zombieBoom;
    Array<Ball> ballArray;
    int type = 1;
    public static boolean outDie = true;
    boolean zombieTouchable;
    int bodyContactNum;
    private boolean dead = false;

    public Zombie(Body body, String name, GamePanel gamePanel, final int type, WorldData boomData) {
        super(gamePanel);
        this.body = body;
        this.gamePanel = gamePanel;
        this.bodyName = name;
        this.type = type;
        body.setUserData(this);
        region = Assets.instance.game.findRegion("heart");
        position = body.getPosition();
        bodyLinearVelocity = body.getLinearVelocity();
        body.destroyFixture(body.getFixtureList().get(0));
        PolygonShape polygonShape = new PolygonShape();
        if (GameInfo.chapter <= 0) {
            zombieTouchable = true;
        } else {
            zombieTouchable = false;
        }
        if (type == 1) {
            if (zombieTouchable) {
                polygonShape.set(new float[]{-0.08f, -0.0f, 0.11f, -0.0f, 0.15f, 0.8f, -0.15f, 0.8f});
                body.setTransform(body.getPosition().x, body.getPosition().y - 0.25f, body.getAngle());
                initZombieAngleLimited();
                initZombieSmokeAnimation();
            } else {
                polygonShape.set(new float[]{-0.1f, -0.25f, 0.2f, -0.25f, 0.2f, 0.55f, -0.1f, 0.55f});
                body.setFixedRotation(true);
            }
        } else if (type == 2) {
            polygonShape.set(new float[]{-0.15f, -0.28f, 0.25f, -0.28f, 0.25f, 0.7f, -0.15f, 0.7f});
        }
        Fixture fixture = body.createFixture(polygonShape, Constant.zombie.density);
        fixture.setFriction(Constant.zombie.friction);
        if (zombieTouchable) {
            fixture.setFilterData(ZombieAttribute.filter);
        } else {
            fixture.setFilterData(ZombieAttribute.filter_no_touch);
        }
        if (type == 1) {
            body.setType(BodyDef.BodyType.DynamicBody);
            zombieAnimation = new StandZombieAnimation();
            body.getFixtureList().get(0).setSensor(false);
        } else if (type == 2) {
            body.setType(BodyDef.BodyType.StaticBody);
            zombieAnimation = new FlyZombieAnimation();
            body.getFixtureList().get(0).setSensor(true);
        }
        if (type == 2 || (type == 1 && !zombieTouchable)) {
            zombieAnimation.setOffset(0.06f, -0.255f);
        }
        zombieBoom = new ZombieBoom(gamePanel, type, boomData);
        if (position.x > 3) {
            zombieAnimation.faceLeft(true);
        } else {
            zombieAnimation.faceLeft(false);
        }
        zombieState = ZombieState.idle;
        zombieAnimation.idle();
        if (jointBody != null) {
            jointBody.setContactListener(new BodyContactAdapter() {
                @Override
                public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
                    super.beginContact(contact, fixtureA, fixtureB, isA);
                    bodyContactNum++;
                }

                @Override
                public void endContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
                    super.endContact(contact, fixtureA, fixtureB, isA);
                    bodyContactNum--;
                }
            });
        }
    }
    public void initZombieAngleLimited(){
        jointBody = gamePanel.createBody(new BodyDef());
        jointBody.setType(BodyDef.BodyType.DynamicBody);
        CircleShape circleShape = new CircleShape();
        circleShape.setRadius(0.1f);
        jointBody.createFixture(circleShape,0.05f);
        jointBody.getFixtureList().get(0).setSensor(true);
        jointBody.setFixedRotation(true);
        jointBody.setTransform(body.getPosition(), 0);
        System.out.println("scene bodies-------->"+gamePanel.getScene());
        RevoluteJointDef revoluteJointDef = new RevoluteJointDef();
        revoluteJointDef.bodyA=body;
        revoluteJointDef.bodyB=jointBody;
        revoluteJointDef.localAnchorA.set(0,0);
        revoluteJointDef.localAnchorB.set(0,0);
        revoluteJointDef.upperAngle = MathUtils.PI/20;
        revoluteJointDef.lowerAngle = -MathUtils.PI/20;
        revoluteJointDef.enableLimit = true;
        gamePanel.getWorld().createJoint(revoluteJointDef);
    }
    private void initZombieSmokeAnimation(){
        if(zombieSomkeAnimation == null){
            zombieSomkeAnimation = new CommonAnimation("animations/smoke.skel",1/160f,"smoke");
            zombieSomkeAnimation.getState().setAnimation(0,"animation",true);
        }
    }


    public void setBallArray(Array<Ball> ballArray) {
        this.ballArray = ballArray;
    }

    public boolean isDead() {
        return dead;
    }

    public void boom(boolean isball, int ballId) {
        dead = true;
        Log.log("Zombie","boom()");
        if(isball) {
            gamePanel.activeBody(body, false);
            if (jointBody != null) {
                gamePanel.activeBody(jointBody, false);
            }
        }
        if(!isball) {
            if (zombieState == ZombieState.out) return;
            zombieState = ZombieState.out;
        }
        else {
            if (zombieState == ZombieState.boom) return;
            zombieState = ZombieState.boom;
            gamePanel.setLastZombieBoomPosition(position);
            boomDung(ballId);
        }
        if(!isball) {
            gamePanel.getScene().checkEnd();
        }
    }

    private void boomDung(int ballId){
        zombieAnimation.boom(ballId);
        gamePanel.activeBody(body, false);
        if(ballId == Constant.BOMB_ID_1|| ballId == Constant.BOMB_ID_2) return;
        Gdx.app.postRunnable(new Runnable() {
            @Override
            public void run() {
                zombieBoom.load(position);
            }
        });
        gamePanel.vibrate();
    }


    public Vector2 getPosition() {
        return position;
    }

    float rotation;
    float angularVelocity;

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        position.set(body.getPosition());
        rotation = body.getAngle() / (float) Math.PI * 180f;
        angularVelocity = body.getAngularVelocity();
        zombieAnimation.update(position.x, position.y,rotation,delta);
        if(zombieSomkeAnimation!=null&&zombieState==ZombieState.smooth) {
            zombieSomkeAnimation.act(delta);
        }
//        checkFaceLeft();
        checkScared();

    }
    public void checkFaceLeft(){
        int leftNum = 0;
        int rightNum = 0;
        for(Ball ball : ballArray){
            if(ball.getBody().getPosition().x>position.x&&ball.getBody().isActive()){
                rightNum++;
            }else {
                leftNum++;
            }
        }
        if(leftNum>=rightNum){
            zombieAnimation.faceLeft(true);
        }else {
            zombieAnimation.faceLeft(false);
        }
    }
    public void checkScared(){
        if(zombieState == ZombieState.boom||zombieState == ZombieState.out) return;
        boolean smooth = false;
        boolean scared = false;
        if(Math.abs(body.getLinearVelocity().x) >= 0.4f){
            smooth = true;
        }else {
            for(Ball ball : ballArray){
                if(!ball.getBody().isActive()) continue;
                if((ball.getBody().getPosition().sub(position).len()<=2)) {
                    scared = true;
                }
            }
        }
        if(smooth){
            if(body.getLinearVelocity().x>0){
                zombieAnimation.faceLeft(false);
            }else {
                zombieAnimation.faceLeft(true);
            }
            if(zombieState != ZombieState.smooth) {
                zombieAnimation.smooth();
                zombieState = ZombieState.smooth;
            }
        }else
            if(scared){
            if(zombieState != ZombieState.scared) {
                zombieAnimation.scared();
                zombieState = ZombieState.scared;
            }
            checkFaceLeft();
        }else {
            if(zombieState != ZombieState.idle) {
                zombieAnimation.idle();
                zombieState = ZombieState.idle;
            }
            checkFaceLeft();
        }

    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
//        if (zombieState == ZombieState.boom || zombieState == ZombieState.out) {
//            zombieBoom.draw(batch, parentAlpha);
//        }
        zombieAnimation.draw(batch);
        zombieSmokeAnimationDraw(batch, parentAlpha);
    }

    protected void zombieSmokeAnimationDraw(Batch batch, float parentAlpha) {
        if(zombieState == ZombieState.smooth && zombieSomkeAnimation!=null && bodyContactNum>0){
            float xOffset = 0;
            float scale = Math.min(bodyLinearVelocity.len2()/6f,0.4f);
            float presentRotation = zombieSomkeAnimation.getRotation();
            float targetRotation;
            if(bodyLinearVelocity.x>0){
                xOffset = -0.08f;
                targetRotation = (360-(bodyLinearVelocity.angle()-180f))%360;
                if(!zombieSomkeAnimation.getSkeleton().getFlipY()) {
                    zombieSomkeAnimation.getSkeleton().setFlipY(true);
                    presentRotation = targetRotation;
                }
            }else {
                xOffset = 0.11f;
                targetRotation = (bodyLinearVelocity.angle()-180f)%360;
                if(zombieSomkeAnimation.getSkeleton().getFlipY()) {
                    zombieSomkeAnimation.getSkeleton().setFlipY(false);
                    presentRotation = targetRotation;
                }
            }
            zombieSomkeAnimation.getSkeleton().getRootBone().setScale(scale);
            zombieSomkeAnimation.setPosition(position.x+xOffset,position.y);
            zombieSomkeAnimation.setRotation(presentRotation+(targetRotation-presentRotation)/4f);
            zombieSomkeAnimation.draw(batch,parentAlpha);
        }
    }

    public void drawZombieBoom(Batch batch, float parentAlpha){
        if (zombieState == ZombieState.boom || zombieState == ZombieState.out) {
            zombieBoom.draw(batch, parentAlpha);
        }
    }


//    public void out(){
//        Log.log("Zombie","out()");
//        if(outDie){
//            gamePanel.end(false);
//        }else{
//            gamePanel.end(true);
//        }
//    }

    public interface ZombieListener {
        public void getPosition(Vector2 position);
    }

    enum ZombieState {
        none, idle, scared, boom,out,smooth
    }

}
