package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.DelayedRemovalArray;
import com.rube.CustomProperties;
import com.rube.Drawer;
import com.rube.ImageData;
import com.rube.ImageUtils;
import com.rube.JsonLoader;
import com.rube.WorldData;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.attributes.ZombieAttribute;
import com.zyb.loveball.utils.MyRandom;

import java.util.HashSet;

public class ZombieBoom extends BaseObject {
    Vector2 position = new Vector2();
    DelayedRemovalArray<Body> bodies = new DelayedRemovalArray<>();
    DelayedRemovalArray<Body> bodiesDefault = new DelayedRemovalArray<>();
    Drawer drawer;
    WorldData data;
    WorldData dataDefault;
    Vector2 tmpVector = new Vector2();
    private ArrayMap<Object, CustomProperties> customProperties;
    protected DelayedRemovalArray<ImageUtils> images = new DelayedRemovalArray<>();

    public ZombieBoom(GamePanel gamePanel, int type,WorldData boomData) {
        super(gamePanel);
        drawer = gamePanel.getDrawer();
        int zombieId = Configuration.settingData.getZombieId();
        customProperties = new ArrayMap<>();
        dataDefault = boomData;
        data = JsonLoader.load("animations/zombieBoom_number_" + zombieId + ".json");
    }

    public void load(Vector2 vector2) {
        position.set(vector2.x, vector2.y);

        JsonLoader.loadImages(data, Assets.instance.game);
        gamePanel.getBox2dLoader().load(gamePanel.getWorld(), data, this.position);
        bodies.addAll(gamePanel.getBox2dLoader().getBodies());
        customProperties.putAll(gamePanel.getBox2dLoader().getCustomProperties());
        loadImages(data,bodies);


        JsonLoader.loadImages(dataDefault, Assets.instance.game);
        gamePanel.getBox2dLoader().load(gamePanel.getWorld(), dataDefault, this.position);
        bodiesDefault.addAll(gamePanel.getBox2dLoader().getBodies());
        customProperties.putAll(gamePanel.getBox2dLoader().getCustomProperties());
        loadImages(dataDefault,bodiesDefault);
        initBodies(bodies);
        initBodies(bodiesDefault);
        removeBody();
        boom(bodies);
        boom(bodiesDefault);
    }

    private void removeBody() {
        HashSet<Integer> bodyInt = new HashSet<>();
        while(bodyInt.size()<(45 - 15)){
            bodyInt.add(MathUtils.random(0,44));
        }
        bodiesDefault.begin();
        for(int index:bodyInt){
            Body tmpBody = bodiesDefault.get(index);
            bodiesDefault.removeValue(tmpBody,true);
            removeImage(tmpBody);
            gamePanel.destroyBody(tmpBody);
        }
        bodiesDefault.end();
/*        Array<BodyData> tmpArray = new Array<>();
        Array<BodyData> bodyDataArray = dataDefault.getBodyData();
        tmpArray.addAll(bodyDataArray);
        for(int index:bodyInt){
            BodyData tmpBody = tmpArray.get(index);
            bodyDataArray.removeValue(tmpBody,true);
        }*/
    }

    private ImageUtils removeImage(Body body) {
        for (int i = 0; i < images.size; i++) {
            if (body == images.get(i).getBody()) {
                ImageUtils imageUtils = images.get(i);
                images.removeIndex(i);
                return imageUtils;
            }
        }
        return null;
    }
    public String getName(Body body) {
        CustomProperties properties = customProperties.get(body);
        if (properties != null) {
            return customProperties.get(body).get("Name");
        } else {
            return null;
        }
    }


    private void initBodies(Array<Body> bodies) {
        for (Body body : bodies) {
            Array<Fixture> fixtureList = body.getFixtureList();
            for (Fixture fixture : fixtureList) {
                if(GameInfo.type==1) {
                    fixture.setFilterData(ZombieAttribute.piecefilter_happyGlass);
                }else {
                    fixture.setFilterData(ZombieAttribute.piecefilter);
                }
                fixture.setRestitution(0.3f);
                fixture.setFriction(0.4f);
            }
        }
    }


    private void boom(Array<Body> bodies) {
        System.out.println("boom!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"+bodies.size);
        for (Body body : bodies) {
            tmpVector.set(0.5f, 0);
            tmpVector.setAngle(MathUtils.random(60, 120));
            tmpVector.scl(MathUtils.random(3, 12));
            float mass = body.getMass();
//            mass = (float) Math.sqrt(mass);
            tmpVector.scl(mass);
//            mass = mass * mass;
            boolean left = MyRandom.getInstance().nextBoolean();
            if (left) {
                body.applyAngularImpulse(mass / 30F, true);
            } else {
                body.applyAngularImpulse(-mass / 30F, true);
            }
            System.out.println(tmpVector);

            body.applyLinearImpulse(tmpVector,body.getWorldCenter(),true);
//            body.applyForceToCenter(tmpVector, true);
        }
    }

    protected void loadImages(WorldData data,Array<Body> bodies) {
        for (int i = 0; i < data.getImageList().size; i++) {
            ImageData temp = data.getImageList().get(i);
            if (temp.getBodyIndex() != -1 && temp.getBodyIndex()<bodies.size) {
                images.add(new ImageUtils(bodies.get(temp.getBodyIndex()), temp));
            }

        }
    }


    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);

        for (ImageUtils imageUtils : images) {
            imageUtils.draw(drawer, batch);
        }
    }
}
