package com.zyb.loveball.screens;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.BaseDialog;
import com.zyb.loveball.dialogs.CoinDialog;
import com.zyb.loveball.dialogs.LimitGoodsDialog;
import com.zyb.loveball.dialogs.SuccessDialog;
import com.zyb.loveball.dialogs.UnlockBallDialog;
import com.zyb.loveball.listeners.FullScreenListener;
import com.zyb.loveball.ui.Grid;
import com.zyb.loveball.utils.CocosUtil;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.LayerUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.SoundButtonListener;

import org.freyja.libgdx.cocostudio.ui.model.CCExport;
import org.freyja.libgdx.cocostudio.ui.parser.group.CCScene;

import java.util.Iterator;
import java.util.Stack;

/**
 * Created by lilexiang on 17-8-15.
 */
public class BaseScreen extends CocosUtil implements Screen {
    protected HelloZombieGame game;
    protected Stage stage;
    protected Actor layer;
    protected Stack<BaseDialog> dialogs = new Stack<>();
    protected String TAG = "";
    protected Group dialogsGroup;
    protected Label coinLabel;
    protected Label starLabel;
    protected boolean screenPaused;
    private PendingAction pendingAction = PendingAction.NONE;
    protected int pendingValue = 0;
    protected ScreenState state;
    protected Class pendingScreen;
    protected float stateTime;
    protected float outTime = 0.3F;
    protected float inTime = 0.3F;
    private Group bannerVideo;
    private int bannerVideoId;
    public boolean showBanner = true;


    public BaseScreen(HelloZombieGame game) {
        this.game = game;
        stage = new Stage(game.getViewport(), HelloZombieGame.getBatch());
    }

    @Override
    public void show() {
        state = ScreenState.show;
        layer = LayerUtils.createLayer();
        loadFromJson(TAG);
        if (scene != null) {
            stage.addActor(scene);
        }
        if (TAG.equals("load")) {
            start();
        } else {
            scene.addActor(layer);
            layer.setColor(0, 0, 0, 1);
            layer.addAction(Actions.sequence(Actions.fadeOut(inTime), Actions.removeActor()));
        }
        Actor bg = findActor("bg");
        if (bg != null && !(this instanceof GameScreen)) {
            Grid grid = new Grid();
            grid.setSize(480, 1000);
            scene.addActorAfter(bg, grid);
        }
        initAnimation();
        Log.log("BaseScreen", "show()" + getClass());
        stateTime = 0;
        if(!Configuration.settingData.isBaseLevelAllPassed()){
            showBanner = false;
            Log.log("BaseScreen","show banner false");
        }
    }

    protected void start() {
        if (state == ScreenState.start) return;
        Log.log("BaseScreen", "start()--->" + getClass() + "   gra:" + Gdx.graphics.getWidth() + "  " + Gdx.graphics.getHeight());
        state = ScreenState.start;
        HelloZombieGame.addInputProcessor(stage);
        HelloZombieGame.resumeInputProcessor();
        initDebug();
        if (layer != null) {
            layer.remove();
        }
        initBanner();
    }


    private void initBanner() {

        if(!showBanner) {
            HelloZombieGame.launcherListener.showBanner(false);
            return;
        }
        if (this instanceof GameScreen && !GameInfo.levelData.isBanner()) {
            HelloZombieGame.launcherListener.showBanner(false);
            return;
        }
        HelloZombieGame.launcherListener.showBanner(true);
//        if (Configuration.banner_state == Configuration.BannerState.none) {
////            if(Configuration.videoAdReady) {
////                HelloZombieGame.launcherListener.showBanner(false);
////                Configuration.banner_state = Configuration.BannerState.video;
////                showBannerVideo();
////            }else{
//                Configuration.banner_state = Configuration.BannerState.ad;
//                HelloZombieGame.launcherListener.showBanner(true);
//                Configuration.banner_time = 5*60;
////            }
//            Log.log("BaseScreen", "showBanner()-----> video, banner time :" + Configuration.banner_time);
//        } else if (Configuration.banner_state == Configuration.BannerState.ad) {
//            HelloZombieGame.launcherListener.showBanner(true);
//            Log.log("BaseScreen", "showBanner()-----> ad, banner time :" + Configuration.banner_time);
//        } else if (Configuration.banner_state == Configuration.BannerState.video) {
//            showBannerVideo();
//            Log.log("BaseScreen", "showBanner()-----> video, banner time :" + Configuration.banner_time);
//        }
//    }
//
//    private void checkBanner() {
//        if(!showBanner) {
//            closeBanner();
//            return;
//        }
//        if (this instanceof GameScreen && !GameInfo.levelData.isBanner()) {
//            closeBanner();
//            return;
//        }
//        if (Configuration.banner_state == Configuration.BannerState.ad) {
//            if (Configuration.banner_time >= Constant.BANNER_AD_TIME && Configuration.videoAdReady) {
//                Configuration.banner_time = 0;
//                HelloZombieGame.launcherListener.showBanner(false);
//                Configuration.bannerVideoInt = null;
//                Configuration.banner_state = Configuration.BannerState.video;
//                Log.log("BaseScreen", "showBanner()-----> video  " + System.currentTimeMillis() / 1000);
//                showBannerVideo();
//            }
//        } else if (Configuration.banner_state == Configuration.BannerState.video) {
//            if (Configuration.banner_time >= Constant.BANNER_VIDEO_TIME || !Configuration.videoAdReady) {
//                Configuration.banner_time = 0;
//                Configuration.banner_state = Configuration.BannerState.ad;
//                HelloZombieGame.launcherListener.showBanner(true);
//                closeBannerVideo();
//                Log.log("BaseScreen", "showBanner()-----> ad  " + System.currentTimeMillis() / 1000);
//            }
//        }
    }

//    private void closeBannerVideo() {
//        if (bannerVideo != null) {
//            bannerVideo.remove();
//        }
//    }

//    private void resetBanner() {
//        Configuration.banner_state = Configuration.BannerState.ad;
//        HelloZombieGame.launcherListener.showBanner(true);
//        Configuration.banner_time = 0;
//        closeBannerVideo();
//    }


//    private void showBannerVideo() {
//        if (bannerVideo != null) {
//            bannerVideo.remove();
//        }
//        final int unlockId = createVideoBanner();
//        bannerVideo.addListener(new LClickListener() {
//            @Override
//            public void touchDownEffect() {
//                super.touchDownEffect();
//                if (bannerVideoId == 0) {
//                    FlurryUtils.Event_BannerClick("Coin200");
//                    playVideoReward(PendingAction.ADD_COIN, 200);
//
//                } else if (bannerVideoId == 1) {
//                    FlurryUtils.Event_BannerClick("SkinFree");
//                    playVideoReward(PendingAction.LIMIT_GOODS, unlockId);
//                } else {
//                    FlurryUtils.Event_BannerClick("SkinDiscount");
//                    playVideoReward(BaseScreen.PendingAction.BALL_UNLOCK, unlockId * Constant.BALLID_PREFIX + (int) (Assets.instance.getGoodData(unlockId, 1).getCoin() * 0.75F));
//                }
////                resetBanner();
//            }
//        });
//    }
//    private void getVideoBannerInt(){
//        int id = Configuration.checkDiscountBallId(0.75F);
//        int randomNum = 3;
//        if (id != -1) {
//            randomNum = 3;
//        } else {
//            randomNum = 2;
//        }
//        final int limitId = Configuration.checkLimitId();
//        int bannerVideoId = MyRandom.getInstance().nextInt(randomNum);
//        if (limitId == -1 && bannerVideoId == 1) {
//            bannerVideoId = 0;
//        }
//        Configuration.bannerVideoInt = new int[]{id,limitId,bannerVideoId};
//    }

//    private int createVideoBanner() {
//        if(Configuration.bannerVideoInt == null){
//            getVideoBannerInt();
//        }
//        int id = Configuration.bannerVideoInt[0];
//        int limitId = Configuration.bannerVideoInt[1];
//        bannerVideoId = Configuration.bannerVideoInt[2];
//        if (bannerVideoId == 0) {
//            FlurryUtils.Event_BannerPush("Coin200");
//            bannerVideo = parseScene("ui/objects/banner_add_coin.json");
//        } else if (bannerVideoId == 1) {
//            FlurryUtils.Event_BannerPush("SkinFree");
//            bannerVideo = parseScene("ui/objects/banner_time_limited_skin.json");
//            Image image = bannerVideo.findActor("skin_mask");
//            Sprite sprite = null;
//            if(limitId<100){
//                sprite = Assets.instance.ui.createSprite(Assets.instance.getGoodData(limitId, 0).getResources()+"_icon");
//            } else if(limitId>100&&limitId<=200) {
//                sprite = Assets.instance.ui.createSprite(Assets.instance.getGoodData(limitId-100, 1).getResources());
//            }else {
//
//            }
//            SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
//            image.setDrawable(spriteDrawable);
//        } else {
//            FlurryUtils.Event_BannerPush("SkinDiscount");
//            bannerVideo = parseScene("ui/objects/banner_discount_skin.json");
//            GoodData goodData = Assets.instance.getGoodData(id,1);
//            ((Label) bannerVideo.findActor("cost_money")).setText("" + goodData.getCoin()*0.75F);
//            ((Label) bannerVideo.findActor("cost_discount")).setText("" + goodData.getCoin());
//            Image image = bannerVideo.findActor("skin_mask");
//            Sprite sprite = Assets.instance.ui.createSprite(goodData.getResources());
//            SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
//            image.setDrawable(spriteDrawable);
//        }
//        bannerVideo.setTouchable(Touchable.enabled);
//        scene.addActor(bannerVideo);
//        bannerVideo.setOrigin(Align.center);
//        bannerVideo.setScale(0.2F);
//        bannerVideo.addAction(Actions.scaleTo(1, 1, 0.1F));
//        if(bannerVideoId == 1){
//            return limitId;
//        }else {
//            return id;
//        }
//    }

    public void playVideoReward(PendingAction action, int pendingValue) {
        setPendingAction(action);
        this.pendingValue = pendingValue;
        HelloZombieGame.launcherListener.playUnityAds();
    }


    public void initAnimation() {
        if (Configuration.poor) return;
        if (!(this instanceof GameScreen)) {
            CommonAnimation potAnimation = new CommonAnimation("animations/pot.skel", 0.7f);
            potAnimation.setColor(1, 1, 1, 0.6F);
            groups[BOTTOM].addActorAfter(findActor("tree"),potAnimation);
            potAnimation.setPosition(338, 75);
            potAnimation.getState().setAnimation(0, "1", true);
        }
    }

    public void loadFromJson(String name) {
        CCExport export = Assets.instance.assetManager.get("ui/" + name + ".json");
        if (export == null) {
            newScene();
        } else {
            try {
                scene = game.getCocoStudioUIEditor().createGroup(export);
            } catch (Exception e) {
                e.printStackTrace();
                newScene();
            }
        }
        scene.setTransform(false);
        scene.setTouchable(Touchable.enabled);
        for (int i = 0; i < GROUP_NAME.length; i++) {
            groups[i] = scene.findActor(GROUP_NAME[i]);
            if (groups[i] == null) {
                groups[i] = new Group();
                scene.addActor(groups[i]);
            }
            groups[i].setTouchable(Touchable.childrenOnly);
            groups[i].setTransform(false);
        }
        dialogsGroup = (Group) findActor("dialogs");
        dialogsGroup.setZIndex(scene.getChildren().size);
        dialogsGroup.setTouchable(Touchable.childrenOnly);
        layout();

        Actor actor = findActor("coin_font");
        if (actor != null) {
            coinLabel = (Label) actor;
        }
        actor = findActor("star_font");
        if (actor != null) {
            starLabel = (Label) actor;
        }
    }

    protected void initDebug() {
        InputAdapter keyAdapter = new InputAdapter() {
            @Override
            public boolean keyUp(int keycode) {
                if (keycode == Input.Keys.ESCAPE || keycode == Input.Keys.BACK) {
                    back();
                }
                if (Configuration.debug || Gdx.app.getType() == Application.ApplicationType.Desktop) {
                    if (keycode == Input.Keys.P) {
                        Gdx.app.postRunnable(new Runnable() {
                            @Override
                            public void run() {
                                MScreenUtil.createLevelPic(game);
                            }
                        });

                    } else if (keycode == Input.Keys.R) {
                        Configuration.resetData();
                    } else if (keycode == Input.Keys.UP) {
                        Configuration.settingData.addCoin(1000);
                        update();
                    } else if (keycode == Input.Keys.NUM_1) {
                        Log.log("BaseScreen","initDebug-------->setBallDeadLine()");
                        Configuration.settingData.setBallDeadline(2);
                    } else if (keycode == Input.Keys.NUM_2) {
                        Log.log("BaseScreen","initDebug-------->setZombieDeadLine()");
                        Configuration.settingData.setZombieDeadline(2);
                    } else if (keycode == Input.Keys.NUM_3) {
                        Log.log("BaseScreen","initDebug-------->setPenDeadLine()");
                        Configuration.settingData.setPenDeadline(2);
                    }
                }
                return super.keyDown(keycode);
            }

        };
        HelloZombieGame.addInputProcessor(keyAdapter);
    }

    private void newScene() {
        if (TAG.equals("load")) return;
        try {
            scene = parseScene("ui/" + TAG + ".json");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setPendingAction(PendingAction action) {
        pendingAction = action;
        game.setPendingActionState(0);
    }

    public void layout() {
        float height = stage.getHeight();
        if (height <= 800) return;
        Actor bg = findActor("bg");
        bg.setHeight(height + 2);
        groups[TOP].setY(height - 800);
        groups[CENTER].setY((height - 800) / 2);
        dialogsGroup.setY((height - 800) / 2F);
    }

    public void update() {
        Log.log("BaseScreen()", "update() --->" + getClass());
        if (game.getPendingActionState() == 1) {
            handlePendingAction(true);
        } else if (game.getPendingActionState() == -1) {
            handlePendingAction(false);
        }
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while (iterator.hasNext()) {
            iterator.next().update();
        }
        if (coinLabel != null) {
            coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        }
        if(starLabel != null){
            starLabel.setText(Configuration.settingData.getStarTotal()+"/"+(Constant.CHAPTER_NUN+Constant.HAPPY_GLASS_CHAPTER_NUN)*30);
//            starLabel.setText(Configuration.settingData.getStarTotal()+"/"+Constant.CHAPTER_NUN*30);
        }
    }

    @Override
    public void dispose() {
        if (stage != null) {
            stage.dispose();
            stage = null;
        }
    }

    int logTime = 0;
    int checkBannerTime;

    @Override
    public void render(float delta) {
        if (stage != null) {
            stage.act();
        }
        if (stage != null) {
            stage.draw();
        }
        if (!screenPaused)
            stateTime += Gdx.graphics.getDeltaTime();
        logTime++;
        if (logTime > 10 * 60) {
            Gdx.app.log("BaseScreen", "render()----> stateTime = " + stateTime + "  state = " + state + "  assetManager update " + Assets.instance.assetManager.update());
            logTime = 0;
        }
        if (state == ScreenState.end && Assets.instance.assetManager.update(10) && stateTime > outTime + 0.02F) {
            state = ScreenState.out;
            out();
        }
        if (state == ScreenState.show && stateTime > inTime + 0.02F) {
            start();
        }
        checkBannerTime++;

        if (state == ScreenState.start && checkBannerTime > 10) {
//            checkBanner();
            checkBannerTime = 0;
        }
        Configuration.banner_time += delta;
    }


    protected void initButtons() {
        Actor btn = findActor("money", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new ClickListener() {
                @Override
                public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                    showCoinDialog();
                    return super.touchDown(event, x, y, pointer, button);
                }
            });
        }
        btn = findActor("btn_back", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new SoundButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    back();
                }
            });
        }
    }

    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass) {
        return showDialog(attribute, tClass, null);
    }


    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass, BaseDialog.DialogListener listener) {
        try {
            BaseDialog dialog = tClass.newInstance();
            Group group = parseScene(attribute);
            dialog.load(group);
            dialog.setBaseListener(getBaseDialogListener());
            dialogs.push(dialog);
            dialog.show(dialogsGroup);
            dialog.update();
            return dialog;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    protected void removeDialog() {
        if (!dialogs.empty()) {
            dialogs.pop();
            update();
        }
    }

    private BaseDialog.DialogListener getBaseDialogListener() {
        BaseDialog.DialogListener dialogListener = new BaseDialog.DialogAdapter() {
            @Override
            public void close() {
                removeDialog();
            }

            @Override
            public void update(boolean updated) {
                BaseScreen.this.update();
            }

            @Override
            public void screenEnd(Class screen) {
                super.screenEnd(screen);
                end(screen);
            }

            @Override
            public void showDialog(String attribute, Class<? extends BaseDialog> tClass) {
                super.showDialog(attribute, tClass);
                BaseScreen.this.showDialog(attribute, tClass);
            }
        };
        return dialogListener;
    }


    public HelloZombieGame getGame() {
        return game;
    }


    protected boolean back() {
        if (dialogs.size() > 0) {
            BaseDialog dialog = dialogs.peek();
            if (dialog != null) {
                dialog.close();
                return false;
            }
        }
        return true;
    }


    @Override
    public void resize(int width, int height) {
        stage.getViewport().update(width, height);
    }

    @Override
    public void hide() {

    }

    @Override
    public void pause() {
        screenPaused = true;
    }


    @Override
    public void resume() {
        update();
        screenPaused = false;
    }

    protected void out() {
        Log.log("BaseScreen", "out() ---> " + pendingScreen);
        if (pendingScreen == MenuScreen.class) {
            game.setScreen(new MenuScreen(game));
        } else if (pendingScreen == GameScreen.class) {
            game.setScreen(new GameScreen(game));
        } else if (pendingScreen == ChapterScreen.class) {
            game.setScreen(new ChapterScreen(game));
        } else if (pendingScreen == LevelScreen.class) {
            game.setScreen(new LevelScreen(game));
        } else if (pendingScreen == ChallengeScreen.class) {
            game.setScreen(new ChallengeScreen(game));
        } else if (pendingScreen == StoreScreen.class) {
            game.setScreen(new StoreScreen(game));
        }
        dispose();
    }

    private void loadResource() {
        if (pendingScreen == ChapterScreen.class) {
            Assets.instance.loadChaptersAtlas();
        } else if (pendingScreen == LevelScreen.class) {
            Assets.instance.loadLevelAtlas(GameInfo.chapter);
        }
    }

    public void end(Class screen) {
        if (state == ScreenState.end) return;
        if (screen == null) return;
        Log.log("BaseScreen", "end()----> " + screen);
//        closeBanner();
        stateTime = 0;
        pendingScreen = screen;
        state = ScreenState.end;
        HelloZombieGame.clearInputProcessor();
        layer.setColor(0, 0, 0, 0);
        stage.addActor(layer);
        layer.getActions().clear();
        layer.addAction(Actions.sequence(Actions.fadeIn(outTime)));
        loadResource();
        checkLimitBall();
    }

    private void checkLimitBall() {
        int ballId = Configuration.settingData.getBallId();
        if (!Configuration.settingData.isGoodUnlocked(ballId, 1)) {
            long deadlineTime = Configuration.settingData.getBallDeadlineTime(ballId);
            long currentTime = System.currentTimeMillis();
            if (deadlineTime - currentTime < 1) {
                Configuration.settingData.setBallId(1);
            }
        }
    }


//    public void closeBanner() {
//        HelloZombieGame.launcherListener.showBanner(false);
//        closeBannerVideo();
//    }


    public void showCoinDialog() {
        CoinDialog coinDialog = (CoinDialog) showDialog("ui/dialogs/coin.json", CoinDialog.class);
        coinDialog.setCoinDialogListener(new CoinDialog.CoinDialogListener() {
            @Override
            public void playVideoAds() {
                setPendingAction(PendingAction.ADD_COIN);
                pendingValue = 200;
                HelloZombieGame.launcherListener.playUnityAds();
            }
        });
    }


    public Group parseScene(String name) {
        return CCScene.parse(game.getCocoStudioUIEditor(), name);
    }

    public enum PendingAction {
        NONE, ADD_COIN, HINT, RESTART, CLEAR, BALL_UNLOCK, REVIVE, SUCCESS_COIN, UNLOCK_LEVEL, LIMIT_GOODS,SUCCESS_BONUS,SECOND_BONUS
    }


    public void handlePendingAction(boolean done) {
        Log.log("BaseScreen", "handle pending action :" + pendingAction + "   done ---> " + done);
        if (!done) setPendingAction(PendingAction.NONE);
        Log.log("BaseScreen", "handle pending action :" + pendingAction + "   done ---> " + done);
        switch (pendingAction) {
            case ADD_COIN:
                coinGot();
                break;
            case BALL_UNLOCK:
                unlockBall();
                break;
            case REVIVE:
                revive();
                break;
            case SUCCESS_COIN:
                successCoinGot();
                break;
            case UNLOCK_LEVEL:
                unlockLevel();
                break;
            case LIMIT_GOODS:
                limitGoods();
                break;
        }
    }




    protected void limitGoods(){
        setPendingAction(PendingAction.NONE);
        FlurryUtils.Ads_Reward("Skin_Limit_Time");
        limitGoods(pendingValue,false);
//        Configuration.settingData.setBallDeadline(pendingValue);
//        LimitGoodsDialog dialog = (LimitGoodsDialog) showDialog("ui/dialogs/banner_unlock_time_limited_skin.json",LimitGoodsDialog.class);
//        dialog.setData(pendingValue);
    }
    protected void limitGoods(int id, boolean isWheel){
        if(id<=100) {
            Configuration.settingData.setPenDeadline(id);
        }else if(id>100&&id<=1000){
            Configuration.settingData.setBallDeadline(id-100);
        }else {
            Configuration.settingData.setZombieDeadline(id-1000);
        }

        LimitGoodsDialog dialog = (LimitGoodsDialog) showDialog("ui/dialogs/banner_unlock_time_limited_skin.json",LimitGoodsDialog.class);
        dialog.setData(id);
    }

    private void unlockLevel() {
        FlurryUtils.Level_Unlock(pendingValue/100,pendingValue%100);
        setPendingAction(PendingAction.NONE);
        System.out.println("unlock "+(pendingValue/100)+"  "+pendingValue%100);
        Configuration.settingData.unlockLevel(pendingValue/100,pendingValue%100,GameInfo.type);
    }

    private void successCoinGot() {
        FlurryUtils.Ads_Reward("Coin");
        setPendingAction(PendingAction.NONE);
        Log.log("BaseScreen", "coin got !!!");
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while (iterator.hasNext()) {
            BaseDialog dialog = iterator.next();
            if (dialog instanceof SuccessDialog) {
                ((SuccessDialog) dialog).extraCoinGot();
            }
        }
    }

    protected void coinGot() {
        setPendingAction(PendingAction.NONE);
        FlurryUtils.Ads_Reward("Coin");
        coinGot(pendingValue,false);
//        Log.log("BaseScreen", "coin got !!!");
//        Configuration.settingData.addCoin(pendingValue);
//        final CommonAnimation coin = new CommonAnimation("animations/jinbi_fly.skel", 0.5F);
//        final Actor actor = LayerUtils.createLayer();
//        dialogsGroup.addActor(actor);
//        actor.addAction(Actions.sequence(Actions.delay(1.7F), Actions.fadeOut(0.3F), Actions.removeActor(), Actions.run(new Runnable() {
//            @Override
//            public void run() {
//                coin.remove();
//            }
//        })));
//        coin.getState().setTimeScale(1);
//        coin.setPosition(0, (Configuration.height - 800) / 2);
//        coin.getState().setAnimation(0, "animation", false);
//        dialogsGroup.addActor(coin);
    }
    protected CommonAnimation coinGot(int coinNum,boolean isWheel){
        Log.log("BaseScreen", "coin got !!!");
        Configuration.settingData.addCoin(coinNum);
        final CommonAnimation coin = new CommonAnimation("animations/jinbi_fly.skel", 0.5F);
        final Actor actor = LayerUtils.createLayer();
        dialogsGroup.addActor(actor);
        actor.addAction(Actions.sequence(Actions.delay(1.7F), Actions.fadeOut(0.3F), Actions.removeActor(), Actions.run(new Runnable() {
            @Override
            public void run() {
                coin.remove();
            }
        })));
        coin.getState().setTimeScale(1);
        coin.setPosition(0, (Configuration.height - 800) / 2);
        coin.getState().setAnimation(0, "animation", false);
        dialogsGroup.addActor(coin);
        return coin;
    }

    private void revive() {
        setPendingAction(PendingAction.NONE);
        GameInfo.life++;
        end(GameScreen.class);
    }

    public void showMessage(String message) {
        Actor temp = dialogsGroup.findActor(message);
        if (temp != null) return;
        Group group = parseScene("ui/objects/text_hint.json");
        group.setY(400);
        Label label = group.findActor("hint_label");
        float ff = label.getX(Align.center);
        label.setWidth(360);
        label.setX(ff, Align.center);
        label.setWrap(true);
        label.setText(message);
        group.setName(message);
        dialogsGroup.addActor(group);
        group.addAction(Actions.sequence(Actions.delay(2), Actions.fadeOut(0.3F), Actions.removeActor()));
    }

    private void unlockBall() {
        FlurryUtils.Ads_Reward("Skin_Discount");
        setPendingAction(PendingAction.NONE);
        int ballId = pendingValue/Constant.BALLID_PREFIX;
        int ballNum = Assets.instance.getGoodsNum(1);
        int cost = pendingValue%Constant.BALLID_PREFIX;
        if(ballId<=0 || ballId>ballNum){
            ballId = Configuration.checkDiscountBallId(0.75F);
        }
        Configuration.settingData.cost(cost);
        UnlockBallDialog dialog = (UnlockBallDialog) showDialog("ui/dialogs/star_unlock_time_limited_skin.json",UnlockBallDialog.class);
        dialog.setData(ballId);
        Log.log("BaseScreen", "unlockBall()-----> " + ballId);
    }

    public static boolean showFullScreenAd(float delay, FullScreenListener listener) {
        if (GameInfo.id <= 8 && GameInfo.chapter == 1) return false;
//        if (Configuration.full_screen_time < 50 && Configuration.retry_times_full_screen < 3) return false;
        if (Configuration.full_screen_time < 40) return false;
        if(!HelloZombieGame.launcherListener.isFullScreenAdReady()) return false;
        HelloZombieGame.launcherListener.showInterstitial(delay,listener);
        Configuration.resetFullScreenTime();
        return true;
    }



    public enum ScreenState {
        show, start, end, dark, out
    }

    public PendingAction getPendingAction() {
        return pendingAction;
    }


}
