package com.zyb.loveball.screens;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.audio.AudioProcess;
import com.zyb.loveball.dialogs.SettingDialog;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.listener.SoundButtonListener;

/**
 * Created by lilexiang on 17-8-15.
 */
public class MenuScreen extends BaseScreen {

    public MenuScreen(HelloZombieGame game) {
        super(game);
        TAG = "menu";
    }

    @Override
    public void show() {
        super.show();
        if(Configuration.settingData.isChapterUnLocked(3)) {
            findActor("btn_start_two").setVisible(true);
        }else {
            findActor("btn_start_two").setVisible(false);
        }
        if(!Configuration.settingData.isLevelUnLocked(1,2,0)){

            findActor("money").setVisible(false);
            findActor("center_hide_group").setVisible(false);
        }
    }

    @Override
    protected void start() {
        super.start();
        initButtons();
        if (Gdx.app.getType() != Application.ApplicationType.iOS) {
            AudioProcess.playMusicLoop("bgm");
        }
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        findActor("btn_start", Touchable.enabled).addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {

                Configuration.resetFullScreenTime();
                GameInfo.normal(0);
                end(GameScreen.class);
                return super.touchDown(event, x, y, pointer, button);
            }
        });
        findActor("btn_start_two", Touchable.enabled).addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                GameInfo.normal(1);
                end(LevelScreen.class);
                return super.touchDown(event, x, y, pointer, button);
            }
        });
//        findActor("btn_challenge", Touchable.enabled).addListener(new SoundButtonListener() {
//            @Override
//            public void touchDownEffect() {
//                super.touchDownEffect();
//                end(ChallengeScreen.class);
//            }
//        });
        findActor("btn_level", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                GameInfo.normal(0);
                end(LevelScreen.class);
            }
        });

        findActor("btn_setting", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                showDialog("ui/dialogs/setting.json", SettingDialog.class);
//                Configuration.settingData.addCoin(10000);
            }
        });

        findActor("btn_skin_game", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                end(StoreScreen.class);
            }
        });
    }

    @Override
    public void layout() {
        super.layout();
        Actor title = findActor("title");
        float titleY = title.getY(Align.center);
        title.setY(titleY*Configuration.height/800,Align.center);
    }

    @Override
    public void initAnimation() {
        super.initAnimation();
        findActor("label_play").setTouchable(Touchable.disabled);
        final CommonAnimation titleAnimation;
        int ballId = Configuration.settingData.getBallId();
        if(ballId == 1) {
            titleAnimation = new CommonAnimation("animations/boom.skel", 0.5F);
        }else {
            titleAnimation = new CommonAnimation("animations/boom"+ballId+".skel", 0.5F);
        }
        groups[BOTTOM].addActor(titleAnimation);
        titleAnimation.getState().setTimeScale(0.6F);
        final float titleY = findActor("title").getY(Align.center);
        titleAnimation.setPosition(305, titleY+128);
        if(Configuration.settingData.isLevelUnLocked(1,2,0)) {
            titleAnimation.addAction(Actions.moveBy(0,-80,1.5F,Interpolation.bounceOut));
            findActor("title").setScale(0.1f);
            findActor("title").addAction(Actions.scaleTo(1,1,0.7f,Interpolation.elasticOut));
        }else {
            titleAnimation.setPosition(305, titleY+48);
        }
        titleAnimation.getState().setAnimation(0, "idle", true);
        if (Configuration.poor) return;
        CommonAnimation animation = new CommonAnimation("animations/zombie"+Configuration.settingData.getZombieId()+".skel",0.5F);
        groups[CENTER].addActor(animation);
//        animation.getState().setTimeScale(0.6F);
        animation.setPosition(240, 465);
        animation.getState().setAnimation(0, "move", true);
        animation.addListener(new ClickListener(){
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                Log.log("MenuScreen","titleAnimation-------> fear!!!");
                titleAnimation.getState().setAnimation(0,"fear",false);
                return super.touchDown(event, x, y, pointer, button);
            }
        });
        findActor("btn_start").addAction(Actions.forever(Actions.rotateBy(360, 19)));
        findActor("label_play").addAction(Actions.forever(Actions.sequence(Actions.scaleTo(1.1f,1.1f,0.5f),Actions.scaleTo(0.95f,0.95f,0.5f))));
    }

    @Override
    public void render(float delta) {
        super.render(delta);
    }

    @Override
    public void dispose() {
        super.dispose();
    }

    @Override
    protected boolean back() {
        boolean flag = super.back();
        if (flag) {
            if(getPendingAction() == PendingAction.NONE) {
                Log.log("MenuScreen", "exit");
                Gdx.app.exit();
            }
        }
        return false;
    }
}
