package com.zyb.loveball.screens;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.ui.StorePanel;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.LClickListener;

import java.text.DecimalFormat;

import javax.swing.GroupLayout;

/**
 * Created by Doodle on 2018/11/7.
 */

public class StoreScreen extends BaseScreen {
    StorePanel ballPanel;
    StorePanel penPanel;
    StorePanel zombiePanel;
    private StoreScreenState storeScreenState = StoreScreenState.ball;

    public StoreScreen(HelloZombieGame game) {
        super(game);
        TAG = "store";
    }

    @Override
    public void show() {
        super.show();
        zombiePanel = new StorePanel(2);
        ballPanel = new StorePanel(1);
        penPanel = new StorePanel(0);
        StorePanel.StorePanelListener listener = new StorePanel.StorePanelListener() {
            @Override
            public void update() {
                StoreScreen.this.update();
            }

            @Override
            public void showCoinDialog() {
                StoreScreen.this.showCoinDialog();
            }
        };
        ballPanel.show(listener, groups[CENTER]);
        penPanel.show(listener, groups[CENTER]);
        zombiePanel.show(listener, groups[CENTER]);
        updateUI();
        initButtons();
        update();
        stage.addListener(new ClickListener());
    }

    private void updateUI() {
        if(storeScreenState == StoreScreenState.ball){
            ((Group) findActor("button")).findActor("btn_ball_bg").setVisible(true);
            ((Group) findActor("button")).findActor("btn_pen_bg").setVisible(false);
            ((Group) findActor("button")).findActor("btn_zombie_bg").setVisible(false);
            ballPanel.setVisible(true);
            penPanel.setVisible(false);
            zombiePanel.setVisible(false);
        }else if(storeScreenState == StoreScreenState.pen){
            ((Group) findActor("button")).findActor("btn_ball_bg").setVisible(false);
            ((Group) findActor("button")).findActor("btn_pen_bg").setVisible(true);
            ((Group) findActor("button")).findActor("btn_zombie_bg").setVisible(false);
            ballPanel.setVisible(false);
            penPanel.setVisible(true);
            zombiePanel.setVisible(false);
        }else if(storeScreenState == StoreScreenState.zombie){
            ((Group) findActor("button")).findActor("btn_ball_bg").setVisible(false);
            ((Group) findActor("button")).findActor("btn_pen_bg").setVisible(false);
            ((Group) findActor("button")).findActor("btn_zombie_bg").setVisible(true);
            ballPanel.setVisible(false);
            penPanel.setVisible(false);
            zombiePanel.setVisible(true);
        }

    }

    @Override
    public void update() {
        super.update();
        penPanel.update();
        ballPanel.update();
        zombiePanel.update();
    }

    @Override
    protected void initButtons() {
        super.initButtons();

        findActor("btn_ball", Touchable.enabled).addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                if (storeScreenState != StoreScreenState.ball) {
                    storeScreenState = StoreScreenState.ball;
                    updateUI();
                }
                return super.touchDown(event, x, y, pointer, button);
            }
        });
        findActor("btn_pen", Touchable.enabled).addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                if (storeScreenState != StoreScreenState.pen) {
                    storeScreenState = StoreScreenState.pen;
                    updateUI();
                }
                return super.touchDown(event, x, y, pointer, button);
            }
        });
        findActor("btn_zombie", Touchable.enabled).addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                if (storeScreenState != StoreScreenState.zombie) {
                    storeScreenState = StoreScreenState.zombie;
                    updateUI();
                }
                return super.touchDown(event, x, y, pointer, button);
            }
        });
        findActor("btn_back", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                back();
            }
        });
    }

    @Override
    protected boolean back() {
        if (super.back()) {
            end(MenuScreen.class);
            return true;
        } else {
            return false;
        }
    }
    protected enum StoreScreenState{
        ball,pen,zombie
    }
}
