package com.zyb.loveball.ui;

import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.Drawable;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.utils.TimeHandle;
import com.zyb.loveball.utils.listener.LClickListener;

/**
 * Created by Doodle on 2018/10/17.
 */

public class BallStoreObject extends StoreObject {
    CommonAnimation animation;
    long limitTime;
    boolean isLimitUse = false;
    TimeHandle timeHandle;
    Group timePanel;
    Label timeLabel;


    public BallStoreObject(Group root, int id, int type, final StoreObjectListener listener) {
        super(root, id, type, listener);
        limitTime = Configuration.settingData.getBallDeadlineTime(id);
        checkLimitUse();
        root.findActor("cost_coin",Touchable.enabled).addListener(new LClickListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if (!listener.isPanning()) {
                    costTouched();
                }
            }
        });
        root.findActor("time_panel").addListener(new LClickListener(){
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if(isLimitUse && Configuration.settingData.getBallId() != BallStoreObject.this.id){
                    Configuration.settingData.setBallId(BallStoreObject.this.id);
                    listener.update();
                }
            }
        });
        initUnlockCondition();
        update();
    }

    private void initUnlockCondition(){
        if(data.getCondition() == 1){
            root.findActor("cost_coin").setVisible(false);
            Group starGroup = root.findActor("cost_star");
            starGroup.setVisible(true);
            Label startLabel = root.findActor("star_label");
            startLabel.layout();
            float width = startLabel.getGlyphLayout().width;
            startLabel.setText(Configuration.numType.format(data.getCoin()));
            startLabel.layout();
            float newWidth = startLabel.getGlyphLayout().width;
            starGroup.setX(starGroup.getX() + (width - newWidth)/2);
        }else{
            root.findActor("cost_coin").setVisible(true);
            root.findActor("cost_star").setVisible(false);
        }
    }

    private void checkLimitUse() {
        timePanel = root.findActor("time_panel");
        timeLabel = timePanel.findActor("time_label");
        long currentTime = System.currentTimeMillis();
        if (!owned && limitTime - currentTime > 10 * 1000) {
            timePanel.setVisible(true);
            isLimitUse = true;
            if(timeHandle == null) {
                timeHandle = new TimeHandle((limitTime - currentTime) / 1000);
            }else{
                timeHandle.reset((limitTime - currentTime) / 1000);
            }
            timeLabel.setText(timeHandle.getInitTimeStr());
            root.getActions().clear();
            root.addAction(Actions.forever(Actions.sequence(Actions.delay(1F), Actions.run(timeRunnable))));
        } else {
            timePanel.setVisible(false);
        }
    }



    Runnable timeRunnable = new Runnable() {
        @Override
        public void run() {
            if (timeHandle != null) {
                timeHandle.update();
                timeLabel.setText(timeHandle.getLeftTime());
                if (timeHandle.isDone()) {
                    timeDown();
                }
            }
        }
    };

    private void timeDown() {
        isLimitUse = false;
//        if (timePanel != null) {
//            timePanel.setVisible(false);
//        }
        if (Configuration.settingData.getBallId() == id) {
            Configuration.settingData.setBallId(1);
//            listener.update();
        }
        listener.update();
    }

    protected void changeMask() {
        mask = root.findActor("mask");
        Sprite sprite = Assets.instance.ui.createSprite(data.getResources());
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        mask.setDrawable(spriteDrawable);

    }

    public void update() {
        owned  = Configuration.settingData.isGoodUnlocked(id, type);
        checkLimitUse();
        if (owned) {
            root.findActor("cost").setVisible(false);
//            root.findActor("select").setVisible(true);
            setSelectVisible(true);
        } else {
            root.findActor("cost").setVisible(true);
//            root.findActor("select").setVisible(false);
            setSelectVisible(false);
        }
        if (owned && id == Configuration.settingData.getGoodId(type)) {
//            root.findActor("using").setVisible(true);
            setUsingVisible(true);
        } else {
//            root.findActor("using").setVisible(false);
            setUsingVisible(false);
        }

        if (!owned && isLimitUse) {
            if (id == Configuration.settingData.getGoodId(type)) {
                ((Label) root.findActor("select_label")).setText("USING");
                root.findActor("time_panel_bg").setVisible(false);
//                root.findActor("using").setVisible(true);
                setUsingVisible(true);
            } else {
                ((Label) root.findActor("select_label")).setText("SELECT");
                root.findActor("time_panel_bg").setVisible(true);
//                root.findActor("using").setVisible(false);
                setUsingVisible(false);
            }
        }

        if (id == Configuration.settingData.getGoodId(type)) {
            initAnimation();
            animation.setVisible(true);
            mask.setVisible(false);
        } else {
            if (animation != null) {
                animation.setVisible(false);
            }
            mask.setVisible(true);
        }
        if(!isLimitUse){
            root.findActor("cost_coin").setY(root.findActor("cost").getHeight()/2,Align.center);
            root.findActor("cost_star").setY(root.findActor("cost").getHeight()/2,Align.center);
        }else {
            root.findActor("cost_coin").setY(0);
            root.findActor("cost_star").setY(7);
        }
//        updateTimePanelBg();
    }

    private void updateTimePanelBg(){
        if(!isLimitUse) return;
        NinePatchDrawable drawable = (NinePatchDrawable)((Image)root.findActor("time_panel_bg")).getDrawable();
        if(id == Configuration.settingData.getGoodId(type)){
            drawable.setPatch(new NinePatch(Assets.instance.ui.findRegion("green_title_bg"),9,9,9,9));
        }else{
            drawable.setPatch(new NinePatch(Assets.instance.ui.findRegion("blue_title_bg"),9,9,9,9));
        }
    }

    private void costTouched(){
        System.out.println("store object " + id + " update" + " current id " + Configuration.settingData.getGoodId(type));
        if (owned) {
            Configuration.settingData.setGoodId(id, type);
        } else {
            if (Configuration.settingData.cost(data.getCoin())) {
                Configuration.settingData.unlockGood(id, type);
            } else {
                listener.showCoinDialog();
            }
        }
        listener.update();
    }



    public void touched() {
        System.out.println("store object " + id + " update" + " current id " + Configuration.settingData.getGoodId(type));
        if (owned) {
            Configuration.settingData.setGoodId(id, type);
            listener.update();
        }
    }


    private void initAnimation() {
        if (animation == null) {
            animation = new CommonAnimation("animations/" + this.data.getAnimation() + ".skel",0.88f);
            root.addActor(animation);
            animation.setPosition(mask.getX(Align.center), mask.getY(Align.center));
            animation.getState().setAnimation(0, "idle", true);
            animation.setVisible(false);
        }
    }

    private void setUsingVisible(boolean visible){
        root.findActor("using").setVisible(visible);
        if(!owned&&isLimitUse){
            if(id == Configuration.settingData.getGoodId(type)) {
                root.findActor("check_mark").setY(root.findActor("time_panel").getY() + root.findActor("check_mark").getHeight() / 2);
                root.findActor("using_font").setVisible(false);
                root.findActor("time_panel").setX(0);
            }else {
                root.findActor("time_panel").setX(11.62f);
            }
        }else {
            root.findActor("check_mark").setY(root.findActor("using_font").getY()+root.findActor("check_mark").getHeight()/2);
            root.findActor("using_font").setVisible(true);
            root.findActor("time_panel").setX(11.62f);
        }
    }

    private void setSelectVisible(boolean visible){
        root.findActor("select").setVisible(visible);
        if(!isLimitUse){
            root.findActor("select").setY(root.findActor("cost").getHeight()/2+15,Align.center);
        }else {
            root.findActor("select").setY(root.findActor("cost_star").getY());
        }
        if(owned){
            root.findActor("select").setY(root.findActor("cost").getHeight()/2+15,Align.center);
        }
    }
}
