package com.zyb.loveball.ui;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.utils.listener.LClickListener;

/**
 * Created by Doodle on 2018/10/17.
 */

public class StoreObject {
    final protected int id;
    protected Group root;
    protected GoodData data;
    protected int type;
//    StorePanel panel;
    protected Image mask;
    protected boolean owned;
    StoreObjectListener listener;

    public StoreObject(Group root, int id, int type, final StoreObjectListener listener) {
        this.id = id;
        this.root = root;
        this.type = type;
        this.listener = listener;
        this.data = Assets.instance.getGoodData(id, type);
        owned = Configuration.settingData.isGoodUnlocked(id, type);
        init();
//        update();
        root.setTouchable(Touchable.enabled);
        root.addListener(new LClickListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if (!listener.isPanning()) {
                    StoreObject.this.touched();
                }
            }
        });
    }

    private void init() {
        Actor bg = root.findActor("net_bg");
        if (bg != null) {
            float bgWidth = bg.getWidth() / 2;
            float bgHeight = bg.getHeight() / 2;
            float centerX = bg.getX() + bg.getWidth() / 2;
            float centerY = bg.getY() + bg.getHeight() / 2;
            Grid grid = new Grid();
            grid.setScale(1.4F);
            grid.setSize((bgWidth - 8) / 1.4F, (bgHeight - 8) / 1.4F);
            grid.setPosition(centerX - grid.getWidth() / 2, centerY - grid.getHeight() / 2);
            root.addActorAfter(root.findActor("net_bg"), grid);
        }
        ((Label) root.findActor("cost_label")).setText(Configuration.numType.format(data.getCoin()));
        changeMask();
    }

    protected void changeMask(){
        mask = root.findActor("mask");
        float centerX = mask.getX() + mask.getWidth() / 2;
        float centerY = mask.getY() + mask.getHeight() / 2;
        TextureRegionDrawable drawable = (TextureRegionDrawable) mask.getDrawable();
        TextureRegion region = null;
        if (type == 0) {
            region = Assets.instance.game.findRegion(data.getResources());
        } else if(type == 1){
            region = Assets.instance.ui.findRegion(data.getResources());
        } else if(type == 2){
            region = Assets.instance.ui.findRegion(data.getResources());
        }
        if (region == null || drawable == null) return;
        drawable.setRegion(region);
        mask.setSize(region.getRegionWidth() / 3, region.getRegionHeight() / 3);
        mask.setPosition(centerX - mask.getWidth() / 2, centerY - mask.getHeight() / 2);
    }

    public void update() {
        owned  = Configuration.settingData.isGoodUnlocked(id, type);
        if (owned) {
            root.findActor("cost").setVisible(false);
            root.findActor("select").setVisible(true);
        } else {
            root.findActor("cost").setVisible(true);
            root.findActor("select").setVisible(false);
        }
        if (id == Configuration.settingData.getGoodId(type)) {
            root.findActor("using").setVisible(true);
        } else {
            root.findActor("using").setVisible(false);
        }
    }

    public void touched() {
        System.out.println("store object " + id + " update" + " current id " + Configuration.settingData.getGoodId(type));
        if (owned) {
            Configuration.settingData.setGoodId(id, type);
        } else {
            if (Configuration.settingData.cost(data.getCoin())) {
                Configuration.settingData.unlockGood(id, type);
            } else {
                listener.showCoinDialog();
            }
        }
        listener.update();
    }


    public interface StoreObjectListener{
        public void update();
        public void showCoinDialog();
        public boolean isPanning();
    }

}
